/*****************************************************************************/
/*                                                                           */
/*                              SOFTING AG                                   */
/*                        Richard-Reitzner-Allee 6                           */
/*                              D-85540 Haar                                 */
/*                      Phone: (++49)-(0)89-45656-0                          */
/*                      Fax:   (++49)-(0)89-45656-399                        */
/*                                                                           */
/*                  Copyright (C) SOFTING AG 1995-2004                       */
/*                                                                           */
/* See file CREDITS for list of people who contributed to this               */
/* project.                                                                  */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    */
/* MA 02111-1307, USA.                                                       */
/*                                                                           */
/* email: support.automation@softing.com                                     */
/* Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany                 */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Author      : SOFTING                                                    */
/*                                                                           */
/*  CHANGE_NOTES                                                             */
/*                                                                           */
/*  date      name      change                                               */
/*  -----------------------------------------------------------------------  */
/*****************************************************************************/
/*****************************************************************************/

#ifndef _CNTRL_H_
#define _CNTRL_H_

//#include <linux/tqueue.h>

#include <linux/interrupt.h>


#define PB_NAME "profiboard"

#ifdef DYNAMIC_MAJOR_NUMBER
  #define PB_MAJOR 0
#else
  #define PB_MAJOR 122
#endif

#define PB_MAX_NR_BOARDS 16



// flags for FilesLoaded
#define PB_B_BOARD_LOADED 0x1
#define PB_UPBOARD_LOADED 0x2
#define PB_FW_LOADED      0x4

#define PB_VERSION_OFFSET 0xc0
#define PB_MASTER_FW    0x00
#define PB_DP_SLAVE_FW  0x01

#define MAX_SBNRECORD_SIZE  4096

typedef enum 
{
  
  PBDATATASK,
  PBFIRSTTASK,
  PBSLAVETASK
      
} active_tasklet;

#if (LINUX_VERSION_CODE < VERSION_CODE(2,6,0))
  typedef void* TASKPARATYPE;
  typedef void IRQRETTYPE;
#define IRQ_NONE
#define IRQ_HANDLED
#else
  typedef unsigned long TASKPARATYPE;
  typedef int IRQRETTYPE;
#endif

#ifndef BOOL
#define BOOL int
#endif

#ifndef FALSE
#define FALSE (0)
#endif

#ifndef TRUE
#define TRUE !FALSE
#endif
  
#define SOFTING_PB_ISA_DEVICE        0x0010
#define SOFTING_PB_PRO_104_DEVICE    0x0023  // PBpro-PC104+  
#define SOFTING_PB_PCI_DEVICE        0x0028
#define SOFTING_PB_PRO_DEVICE        0x002A
#define SOFTING_PB_PCIE_DEVICE       0x002B


typedef struct __wait_struct__
{
  volatile int counter;
  spinlock_t lock;
  char *name;
  
} WAIT_STRUCT;

struct CntrlExt 
{
 // struct tq_struct PbDataISTQueue;
  struct tasklet_struct PbDataTasklet;
  struct tasklet_struct PbFirstTasklet;
  struct tasklet_struct PbSlaveTasklet;
  active_tasklet active_tsklt;
  spinlock_t Spinlock;
  unsigned short insideSpinlock;
  unsigned short asic_type;
  BOOL isUsed;
  unsigned short DeviceType;
  volatile unsigned short volatile *c_to_h_irq_val;
  volatile unsigned short volatile *h_to_c_irq_val;
  PB_ADDRESS physicalDPRAM;
  PB_ADDRESS config_space_mem;
  u32           asic;
  
  unsigned char SbnBuffer[MAX_SBNRECORD_SIZE];
  unsigned short fwBufferPtr;
  BOOL           isFwLoading;
  
  
  unsigned int port;
  unsigned char BoardCtrl;
  unsigned char interrupt;
  PB_ADDRESS RealDprAddr;
  PB_ADDRESS DprAddr;
  PB_ADDRESS ConfigBase;
  PB_ADDRESS h_data_descr_addr;
  unsigned long Channel;
  void *HSdbStart;
  void *HSdbBuffer;
  int HSdbDataLength;
  void *CSdbStart;
  int CSdbDataLength;
  unsigned long NrIrqs;
  char *FWVersion;
  int SerialNumber;         // 4 bytes
  int TypeOfFW;
  struct _Board 
  {
    int InUseCountRead;
    int InUseCountWrite;
  } Board;
  struct _Service {
    int CMIClosed;
    int Exception; // set, when an exception interrupt occurs
    int Calls;
    int CallsPerSecond;
    int Max;
    int InUseCountRead;
    int InUseCountWrite;
    wait_queue_head_t QRead;
    wait_queue_head_t QWrite;
    WAIT_STRUCT wait_for_read;
    WAIT_STRUCT wait_for_write;
  } Service;
  struct _Data 
  {
    int Calls;
    int CallsPerSecond;
    int Max;
    USIGN32 Cycles;
    int InUseCountRead;
    int InUseCountWrite;
    int OptimizedMode;     // Read will block until data changed
    int NewInputData;      // Data changed, really new data arrived
    wait_queue_head_t QNewInputData;
    WAIT_STRUCT wait_for_new_input_data;
  } Data;
  struct _SlaveIn {
    int InUseCountRead;
    int InUseCountWrite;
    void *InStruct;
    wait_queue_head_t QInputAtController;
    unsigned char ImageIdx;
  } SlaveIn;
  
  struct _SlaveOut 
  {
    int InUseCountRead;
    int InUseCountWrite;
    USIGN32 Cycles;
    void *OutStruct;
    // XXX - don't like to include pb_xxx.h here, Geeek.
    struct 
    {
      unsigned char output_data_len;
      unsigned char output_state;
      unsigned char output_data[244];
      unsigned char output_state_copy;
      unsigned char output_data_copy[244];
    } Cache;
  } SlaveOut;
  long timeout;              // in jiffies
  
  struct _FwDwnld 
  {
    int FilesLoaded;       // 00000111 Bit for b_board, upboard and fw
    int State;             // reading tar header or parts of the fw files
    int GapSize;           // bytes between the last file and the next block
    USIGN32 FileSize;// remaining bytes in the file
    char *TarHeader;       // beginning of the header area (kmalloc)
    char *TarHeaderAct;    // actual buffer end
    char *Firmware;        // beginning of the fw areas (DPRAM)
    char *FirmwareAct;
    PB_ADDRESS StartAddr;// StartAddress of SBIN file
    unsigned short PrevCheckSum;// Checksum of last SBIN file
    int count;              // count of records; just for debugging ...
  } FwDwnld;
  
  
  struct _dpdata 
  {
    PB_ADDRESS InAddr;
    PB_ADDRESS OutAddr;
    PB_ADDRESS StateAddr;
    unsigned char *InBuf;
    unsigned char *OutBuf;
    unsigned char *StateBuf;
    unsigned char *InBufCopy;
    unsigned char *OutBufCopy;
    unsigned char *StateBufCopy;
    int InLength;
    int OutLength;
    int InOffset;
    int OutOffset;
    int StateLength;
    int dpState;
    int dpStatePending;
  } Dp;
};

extern struct CntrlExt CtrlExt[PB_MAX_NR_BOARDS];

//-----------------------------------------------------------------------------
// defines

// common memory ready masks
#define H_READY_MASK  0x874B2D1E  // host ready mask
#define C_READY_MASK  0x78B4D2E1  // controller ready mask

// communication mode
#define INIT_MODE 0x00  // initializing  mode
#define CONFIG_MODE 0x05  // configuration mode
#define COMM_MODE 0x08  // communication mode

// semaphore for service interface access
#define IDLE  0x00  // semaphore idle
#define BUSY  0x01  // semaphore busy

// semaphore for data interface access
#define D_DATA_IDLE   0x00  // data semaphore idle
#define D_DATA_APPLY  0xF0  // data semaphore apply
#define D_DATA_BUSY   0x0F  // data semaphore busy

// host identifiers
#define CMI_V300  0xA0  // host driver CMI version 3.0
#define CMI_V310  0xA1  // host driver CMI version 3.1

// controller identifier
#define ID_PROFIBOARD 0xCA  // PROFIboard
#define ID_PROFICARD  0xBA  // PROFIcard
#define ID_H1_V2    0xAA  // H1_V2 controller board

// write ports
#define BOARD_CTRL  0 // Board Control Register
#define BASE_ADD  1 // DPRAM Base Address Register
#define PAGE_SEL  2 // Page Select Register
#define CP_IRQ    3 // Interrupt Register

// Pindefinition of the Page Select Register
#define PAGE_A0      0x01 // Page Address low within the 64 kByte window
#define PAGE_A1      0x02 // Page Address high within the 64 kByte window

#define PAGE_0       0x00 // Page 0  0KB - 16KB
#define PAGE_1       0x01 // Page 1 16KB - 32KB
#define PAGE_2       0x02 // Page 2 32KB - 48KB
#define PAGE_3       0x03 // Page 3 48KB - 64KB
#define EN_IRQ3      0x04 // Enable Interrupt Request 3
#define EN_IRQ5      0x08 // Enable Interrupt Request 5
#define EN_IRQ10     0x10 // Enable Interrupt Request 10
#define EN_IRQ11     0x20 // Enable Interrupt Request 11
#define EN_IRQ12     0x40 // Enable Interrupt Request 12
#define EN_IRQ15     0x80 // Enable Interrupt Request 15

// Pindefinition of the Board Control Register
#define C165_RESET      0x01  // Reset of the C165 (low active)
#define V25_RESET       0x01  // Reset of the V25 (low active)
#define EN_DPRAM        0x02  // Enable DPRAM for PC (high active)
#define SIZE_16_64      0x04  // DPRAM size 1:16 kByte 0:64kByte
#define BASE_A14        0x08  // Address low within the 64 kByte
#define BASE_A15        0x10  // Address high within the 64 kByte
#define DPR_WIDTH_8_16  0x20  // DPRAM width 0:16Bit 1:8Bit
#define MEMCS16_LA_SA   0x40  // MEMCS16 generation 0:LA 1:SA
#define RES_IRQ_V_P   0x80  // Reset Interrupt Request from V25 (low activ)
#define DPRAM_SIZE_16KB DPR_PAGE_SIZE

// DPR page size and page offsets
#define DPR_PAGE_SIZE   0x4000

#define PAGE1_OFFS      0x4000
#define PAGE2_OFFS      0x8000
#define PAGE3_OFFS      0xC000

// position of chip select (only PROFIcard)
#define H_CHIP_SELECT_OFFSET      0x3FFA  // chip select offset

// host identifiers
#define CMI_V300            0xA0   // host driver CMI version 3.0
#define CMI_V310            0xA1   // host driver CMI version 3.1
#define CMI_V410            0xA1   // CMI version 4.10 FMS/DPV1-Master
#define CMI_V410_DP_SLAVE   0xB1   // CMI version 4.10 DP-Slave

#define DPR_SIZE                      0x4000  // DPR size
#define H_TO_C_IRQ_VAL_OFFSET         0x3FFE  // host to controller irq value offset
#define C_TO_H_IRQ_VAL_OFFSET         0x3FFC  // controller to host irq value offset
#define H_TO_C_IRQ_VAL_OFFSET_PRO     0x1FFFC  // host to controller irq value offset
#define C_TO_H_IRQ_VAL_OFFSET_PRO     0x1FFFE  // controller to host irq value offset
#define C_TO_H_IRQ_VAL_OFFSET_PCIE    0x7FFFE
#define H_TO_C_IRQ_VAL_OFFSET_PCIE    0x7FFFC

extern void irq_to_cntrl( struct CntrlExt *CtrlExt, unsigned char IrqValue);
extern int SleepAWhile( int *Timeout );
extern void /*inline*/ StartTimeout( int *Timeout, int value);
extern void PbStartCntrl( struct CntrlExt *Board );
extern int WaitForBootFlag( struct CntrlExt *Board );
void DpramOut( struct CntrlExt *Board, int offset );

extern void ReleaseSpinlock(struct CntrlExt *Board, unsigned long Flags, unsigned long line);
extern void GetSpinlock(struct CntrlExt *Board, unsigned long *Flags, unsigned long line);

extern void init_my_wait(WAIT_STRUCT *pWait, char *name);
extern void get_wait(WAIT_STRUCT *pWait);
extern void free_wait(WAIT_STRUCT *pWait);



#endif
