/*****************************************************************************/
/*                                                                           */
/*                              SOFTING AG                                   */
/*                        Richard-Reitzner-Allee 6                           */
/*                              D-85540 Haar                                 */
/*                      Phone: (++49)-(0)89-45656-0                          */
/*                      Fax:   (++49)-(0)89-45656-399                        */
/*                                                                           */
/*                  Copyright (C) SOFTING AG 1995-2004                       */
/*                                                                           */
/* See file CREDITS for list of people who contributed to this               */
/* project.                                                                  */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    */
/* MA 02111-1307, USA.                                                       */
/*                                                                           */
/* email: support.automation@softing.com                                     */
/* Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany                 */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Author      : SOFTING                                                    */
/*                                                                           */
/*  CHANGE_NOTES                                                             */
/*                                                                           */
/*  date      name      change                                               */
/*  -----------------------------------------------------------------------  */
/*****************************************************************************/
/*****************************************************************************/

#ifndef FW_H
#define FW_H

#include "basetypes.h"
#include "cntrl.h"



// This numbers are really magic!
//
//#define TIMEOUT_CARD_CLEAR_FLASH    190000
//#define TIMEOUT_CARD                 25000
#define TIMEOUT_CARD_CLEAR_FLASH    25000
#define TIMEOUT_CARD                10000

// offset of firmware
#define OFFSET_FIRMWARE_STRING 0xC0L
#define FIRMWARE_STRING_LEN    0x40L
// offset for bootblock check
#define OFFSET_BOOT_BLOCK_FLAG 0x70L        // Offset for DPRAM entrie
// bootblock flags
#define FLAG_BOOT_BLOCK      0x544F4F42L    // "BOOT" Bootblock-Flag
#define FLAG_NO_BOOT_BLOCK   0x42424F4EL    // "NOBB" No Bootblock-Flag

// offsets for firmware download procedure
#define OFFSET_COMMAND_CODE 0x7e0L
#define OFFSET_COMMAND_DATA 0x7e2L
#define OFFSET_RECEIPT_CODE 0x7f0L
#define OFFSET_RECEIPT_DATA 0x7f2L


#define OFFSET_BOOT_IMAGE       0x0
#define ADDR_OFFSET_BOOT_IMAGE  0x0
#define OFFSET_LOAD_IMAGE       0x0800
#define ADDR_OFFSET_LOAD_IMAGE  0x080000

#define OFFSET_DNL_DATA     0x100

#define ADDR_DPR_FROM_BOARD_VIEW    0x120000L
#define ADDR_DPR_FROM_CARD_VIEW 0xd0000L

// commands
#define COMMAND_NONE        0x00
#define COMMAND_COPY_MEM    0x01
#define COMMAND_FILL_MEM    0x02
#define COMMAND_JUMP_PROG   0x03
#define COMMAND_START_TEST  0x10

// receipts
#define RECEIPT_NONE            0x00
#define RECEIPT_OK              0x01
#define RECEIPT_NOT_OK          0x02
#define RECEIPT_COMMAND_UNKNOWN 0x03
#define RECEIPT_ERR_FLASH_MANUF 0x04
#define RECEIPT_NOT_SUP_MANUF   0x05
#define RECEIPT_ERR_AMD_TYPE    0x06
#define RECEIPT_ERR_AMD_TOP     0x07
#define RECEIPT_WRITE_DOOTBLOCK 0x08
#define RECEIPT_TEST_MSG        0x10


#define SBIN_TYPE_STRING    0xffff
#define SBIN_TYPE_DATA      0x0000
#define SBIN_TYPE_EOF       0x0001
#define SBIN_TYPE_START     0x0003

#define SBIN_LABEL "Structured Binary Format, Softing GmbH"

// the complete packet: header + data + checksum
#define MAX_SBIN_DATA_SIZE (__u16)(OFFSET_COMMAND_CODE - OFFSET_DNL_DATA)
#define MAX_SBIN_RECORD_SIZE (__u16)(MAX_SBIN_DATA_SIZE+sizeof(SBinHeader)+2)

typedef struct _SBinHeader {
  __u16 type;
  __u32 addr;
  __u16 len;
}  __attribute__ ((packed)) SBinHeader;

typedef struct _Copymem {
  __u32 sourceAddrAbs;
  __u32 destinationAddrAbs;
  __u16 len;
  __u8 calcCS;
} __attribute__ ((packed)) Copymem;

typedef struct _Jumpprog {
  __u32 entryPointsAbs;
  __u8 autoRestart;
} __attribute__ ((packed)) Jumpprog;

typedef struct _ReceiptCopyMem {
  __u16 CheckSum;
} __attribute__ ((packed)) ReceiptCopyMem;

int PbInitBoard( struct CntrlExt *Board, char Reset );
int PbStartBoard( struct CntrlExt *Board, char Reset );
int CheckForBootblock( struct CntrlExt *Board );

#endif
