/*****************************************************************************/
/*                                                                           */
/*                              SOFTING AG                                   */
/*                        Richard-Reitzner-Allee 6                           */
/*                              D-85540 Haar                                 */
/*                      Phone: (++49)-(0)89-45656-0                          */
/*                      Fax:   (++49)-(0)89-45656-399                        */
/*                                                                           */
/*                  Copyright (C) SOFTING AG 1995-2004                       */
/*                                                                           */
/* See file CREDITS for list of people who contributed to this               */
/* project.                                                                  */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    */
/* MA 02111-1307, USA.                                                       */
/*                                                                           */
/* email: support.automation@softing.com                                     */
/* Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany                 */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Author      : SOFTING                                                    */
/*                                                                           */
/*  CHANGE_NOTES                                                             */
/*                                                                           */
/*  date      name      change                                               */
/*  -----------------------------------------------------------------------  */
/*****************************************************************************/
/*****************************************************************************/

#include <linux/fs.h>
#include <linux/mm.h>
#include <asm/param.h> // for HZ
#include <asm/io.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/delay.h>
#include "kernel_version.h"

#if (VERSION_CODE(2,2,0) <= LINUX_VERSION_CODE)
  #include <linux/poll.h>
#endif

#include "basetypes.h"
#include "pb_ioctl.h"
#include "cntrl.h"
#include "cmi.h"
#include "pb_debug.h"
#include "xx_if.h"

extern void DpramOut( struct CntrlExt *Board, int offset );
extern void PbDataIST( void *Args );
extern void ReleaseSpinlock(struct CntrlExt *Board, unsigned long Flags, unsigned long line);
extern void GetSpinlock(struct CntrlExt *Board, unsigned long *Flags, unsigned long line);
extern void leaveGlobMutex(void);
extern int enterGlobMutex(void);
extern void DumpMem(void *pMem, unsigned long size, char *pText);




static int ReadStateAndData(struct CntrlExt *Board, struct file *file, StatusAndData *Status)
{
  int count = Status->DataLength, offset, StatusLength;
  unsigned long IrqFlags;
  unsigned long dummy;

  /***************/
  /* Data Device */
  /***************/
  Board->Data.Calls++;
  offset = file->f_pos - Board->Dp.InOffset;
  if( count > Board->Dp.InLength ) 
  {
    count = Board->Dp.InLength;
  }
  PR_DEBUG(D_DATA_L1,"ioctl state/data %d bytes from %ld\n",
      count,(long)file->f_pos);
  if( (count+offset) > Board->Dp.InLength ) 
  {
    count = Board->Dp.InLength - offset;
    if( count < 0 )
      count = 0;
  }
  StatusLength = Status->StatusLength > Board->Dp.StateLength ?
    Board->Dp.StateLength : Status->StatusLength;
 // cli(); // if the ISR writes new data at the time we are copying ...
  
  if( Board->Dp.InBuf && count ) 
  {
    GetSpinlock(Board, &IrqFlags, __LINE__);
    
    Board->Data.NewInputData = 0;
    memcpy (Board->Dp.StateBufCopy, Board->Dp.StateBuf,StatusLength);
    memcpy (Board->Dp.InBufCopy, Board->Dp.InBuf+offset, count);
    
    ReleaseSpinlock(Board, IrqFlags, __LINE__);
    
    dummy = copy_to_user (Status->StatusImage, Board->Dp.StateBufCopy,StatusLength);
    dummy = copy_to_user (Status->Data, Board->Dp.InBufCopy, count);
    
    printk("InBufCopy: %p: 0x%02x 0x%02x 0x%02x 0x%02x \n", 
           &Board->Dp.InBufCopy[0],
           ((unsigned char *)(Board->Dp.InBufCopy))[0], 
           ((unsigned char *)(Board->Dp.InBufCopy))[1], 
           ((unsigned char *)(Board->Dp.InBufCopy))[2], 
           ((unsigned char *)(Board->Dp.InBufCopy))[3]);
    
    Status->StatusLength = StatusLength;
    Status->DataLength = count;
  } 
  else
  {
    Status->StatusLength = 0;
    Status->DataLength = 0;
  }
  
 // sti();
  return( count );
}

static int WriteInputGetState( struct CntrlExt *Board, struct file *file,
    NTIoctl *Args )
{
  int ret;

  if((ret=cmi_set_dps_input_data( Board, (uint16)Args->InBufLength,
      (uint8*)Args->InBuf, (uint8*)Args->OutBuf))==0 ) 
  {
    Args->OutBufLength = 1;
    return( 0 );
  } 
  else 
  {
    printk(KERN_ERR "WriteInputGetState: %d\n", ret );
    Args->OutBufLength = 1;
  }
  return( ret );
}

/*********/
/* IOCTL */
/*********/

#define PRINT 

#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,32)
long pb_ioctl( struct file *file, unsigned int cmd, unsigned long arg ) 
#else
int pb_ioctl( struct inode *inode, struct file *file, unsigned int cmd,
              unsigned long arg )
#endif
{
  struct CntrlExt *Board = file->private_data;
  StatusAndData Status;
  NTIoctl ntargs;
  DataImage image;
  unsigned long dummy;
  unsigned long flags;
  
  unsigned char buffer[256];
  int err;
 // int16 ret;

  // printk("PbIoctl 0x%x\n", cmd);
  PR_DEBUG(D_ALL_L1,"PbIoctl 0x%x\n", cmd);
  
  if(enterGlobMutex())
  {
    return (-EINTR);
  }

  //if( !arg ) return( -EINVAL ); // arg can be 0!
  switch( cmd ) 
  {
    case IOCTL_PROFI_SET_TIMEOUT:
      Board->timeout = (long) arg*HZ/1000; // arg in ms, we need jiffies
      PR_DEBUG(D_ALL_L1,"profiboard: timeout=%ld\n", Board->timeout );
      break;
    case IOCTL_PROFI_OPTIMIZED_MODE:
      Board->Data.OptimizedMode = (int) arg;
      PR_DEBUG(D_DATA_L3, "profiboard: data dev in %s mode!\n",
               Board->Data.OptimizedMode ? "OPTIMIZED" : "NORMAL" );
      break;
    case IOCTL_PROFI_SERIAL_NUMBER:
      if(!(err=access_ok( VERIFY_WRITE, (void *)arg, sizeof(int *)) ) ) 
      {
      //PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
    //copy_to_user
      printk("serial number: %u\n", Board->SerialNumber);
//       put_user( Board->SerialNumber, (int *) arg );
      dummy = copy_to_user ((void *)arg, &Board->SerialNumber, sizeof(Board->SerialNumber));
      break;
    case IOCTL_PROFI_GET_DPRAM:
      if (((void*) arg == NULL) ||
            (!(err=access_ok( VERIFY_READ, (void *)arg, sizeof(ntargs)))) ) 
      {
      //PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
      PR_DEBUG(D_ALL_L1, "verify_area %lp READ returned %d\n", (void*) arg, err);
      dummy = copy_from_user (&ntargs, (void *)arg, sizeof(ntargs));
      PR_DEBUG(D_ALL_L1, "PROFI_GET_DPRAM %d\n", ntargs.OutBufLength );
      if(!(err=access_ok( VERIFY_WRITE, (void *)ntargs.OutBuf,
           ntargs.OutBufLength) ) ) 
      {
      //PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
      PR_DEBUG(D_ALL_L1, "verify_area %lp WRITE returned %d\n", (void*) ntargs.OutBuf, err);
//    if( Board->Dp.InLength < ntargs.OutBufLength )
//      ntargs.OutBufLength = Board->Dp.InLength;
      if( (file->f_pos+ntargs.OutBufLength) > DPRAM_SIZE_16KB ) 
      {
        ntargs.OutBufLength = DPRAM_SIZE_16KB - file->f_pos;
        if( ntargs.OutBufLength < 0 )
          ntargs.OutBufLength = 0;
      }
    //DpramOut( Board, 0 );
      PR_DEBUG(D_ALL_L1, "copy from io %lp to %lp\n", (void*) Board->DprAddr,
               (void*) ntargs.OutBuf);
      copy_from_io ((void*) ntargs.OutBuf,(void *)Board->DprAddr,ntargs.OutBufLength);
      if(!(err=access_ok( VERIFY_WRITE, (void *)arg, sizeof(ntargs)) ) ) 
      {
     // PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
      PR_DEBUG(D_ALL_L1, "verify_area %lp WRITE returned %d\n", (void*) arg, err);
      dummy = copy_to_user ((void *)arg, &ntargs, sizeof(ntargs));
      PR_DEBUG(D_ALL_L1, "PROFI_GET_DPRAM returned %d\n",
               ntargs.OutBufLength );
      break;
    case IOCTL_PROFI_STATUS_DATA:
      if(!(err=access_ok( VERIFY_WRITE, (void *)arg, sizeof(Status)) ) ) 
      {
       // PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
      dummy = copy_from_user (&Status, (void *)arg, sizeof(Status));
      PR_DEBUG(D_DATA_L1,"Status: %p/%d, Data: %p/%d\n",
               Status.StatusImage, Status.StatusLength,
               Status.Data, Status.DataLength );
    // check parameters
      if(!(err=access_ok( VERIFY_WRITE, (void *)Status.StatusImage,
           Status.StatusLength)) )
      {
     // PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
      if(!(err=access_ok( VERIFY_WRITE, (void *)Status.Data,
           Status.DataLength)) )
      {
     //PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
      ReadStateAndData( Board, file, &Status );
      dummy = copy_to_user ((void *)arg, &Status, sizeof(Status));
      break;
    case IOCTL_PROFI_SET_SLAVE_IN:
      if(!(err=access_ok( VERIFY_WRITE, (void *)arg, sizeof(ntargs)) ) ) 
      {
        PR_DEBUG(D_SLAVE_L1,"ioctl_profi_set_slave_in: access_not_ok\n");
     // PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
      dummy = copy_from_user (&ntargs, (void *)arg, sizeof(ntargs));
      PR_DEBUG(D_SLAVE_L1,"ioctl_profi_set_slave_in: %d/%d\n",
               ntargs.InBufLength, ntargs.OutBufLength );
    // check parameter
      if(!(err=access_ok( VERIFY_WRITE, (void *)ntargs.OutBuf,
           ntargs.OutBufLength)) )
      {
     // PRINT("PbIoctl  line %u\n",__LINE__);
        leaveGlobMutex();
        return( err );
      }
      if(!(err=access_ok( VERIFY_READ, (void *)ntargs.InBuf,
           ntargs.InBufLength)) )
      {
     // PRINT("PbIoctl  line %u\n",__LINE__);
        leaveGlobMutex();
        return( err );
      }
      dummy = copy_from_user (buffer, ntargs.InBuf, ntargs.InBufLength);
      PR_DEBUG(D_SLAVE_L1,"data: 0x%x 0x%x 0x%x 0x%x 0x%x\n",
               buffer[0], buffer[1], buffer[2], buffer[3], buffer[4] );
      err=WriteInputGetState( Board, file, &ntargs );
      dummy = copy_to_user ((void *)arg, &ntargs, sizeof(ntargs));
      if( err!=E_OK ) 
      {
     // PRINT("PbIoctl  line %u\n",__LINE__);
        leaveGlobMutex();
        return (err);
      }
      break;
    case IOCTL_PROFI_DP_OPERATE:
      if(!(err=access_ok( VERIFY_WRITE, (void *)arg, sizeof(int *)) ) ) 
      {
      //PRINT("PbIoctl  line %u\n",__LINE__);
        leaveGlobMutex();
        return( -EINVAL );
      }
    // put_user((Board->PbDataISTQueue.routine == PbDataIST), (int *)arg);
      put_user((Board->active_tsklt == PBDATATASK), (int *)arg);    
    
      break;
    case IOCTL_PROFI_SET_DEBUGLEVEL:
      DebugArea = (unsigned int) arg;
   // printk("PbIoctl(SET_DEBUGLEVEL)= 0x%x\n", DebugArea );
      break;
    case IOCTL_PROFI_GET_DATA_IMAGE:
      PR_DEBUG(D_ALL_L1, "ioctl_profi_get_data_image\n");

      if(!(err=access_ok(VERIFY_WRITE,(void *)arg,sizeof(image)))) 
      {
        leaveGlobMutex();
        return( -EINVAL );
      }
      dummy = copy_from_user (&image, (void *)arg, sizeof(image));
        
//       printk("dummy=%u, size=%u, len=%u\n",dummy, sizeof(image),image.Length);
      
          // check parameter
      if(!(err=access_ok( VERIFY_WRITE, (void *)image.Buf, image.Length)) )
      {
        leaveGlobMutex();
        return( err );
      }
     // printk("length=%u, offset=%lu\n",image.Length,file->f_pos);
      GetSpinlock(Board, &flags, __LINE__);
      memcpy((char*)Board->Dp.InBufCopy, (char*)Board->Dp.InBuf+file->f_pos, image.Length);
      ReleaseSpinlock(Board, flags, __LINE__);
      dummy = copy_to_user(image.Buf, (char*)Board->Dp.InBufCopy, image.Length);
      
//       printk("InBufCopy: line %u, %p: 0x%02x 0x%02x 0x%02x 0x%02x f_pos=%llu, len=%u\n", 
//              __LINE__,
//              &Board->Dp.InBufCopy[0],
//              ((unsigned char *)(Board->Dp.InBufCopy))[0], 
//              ((unsigned char *)(Board->Dp.InBufCopy))[1], 
//              ((unsigned char *)(Board->Dp.InBufCopy))[2], 
//              ((unsigned char *)(Board->Dp.InBufCopy))[3],
//              file->f_pos,
//              image.Length
//             );
      
      
      break;
    default:
    {
   // PRINT("PbIoctl -EINVAL line %u\n",__LINE__);
      leaveGlobMutex();
      return( -EINVAL );
    }
  }
  // printk("PbIoctl Ok! line %u\n",__LINE__);
  leaveGlobMutex();
  return( 0 );
}
