/*****************************************************************************/
/*                                                                           */
/*                              SOFTING AG                                   */
/*                        Richard-Reitzner-Allee 6                           */
/*                              D-85540 Haar                                 */
/*                      Phone: (++49)-(0)89-45656-0                          */
/*                      Fax:   (++49)-(0)89-45656-399                        */
/*                                                                           */
/*                  Copyright (C) SOFTING AG 1995-2004                       */
/*                                                                           */
/* See file CREDITS for list of people who contributed to this               */
/* project.                                                                  */
/*                                                                           */
/* This program is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU General Public License as            */
/* published by the Free Software Foundation; either version 2 of            */
/* the License, or (at your option) any later version.                       */
/*                                                                           */
/* This program is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             */
/* GNU General Public License for more details.                              */
/*                                                                           */
/* You should have received a copy of the GNU General Public License         */
/* along with this program; if not, write to the Free Software               */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston,                    */
/* MA 02111-1307, USA.                                                       */
/*                                                                           */
/* email: support.automation@softing.com                                     */
/* Softing AG, Richard-Reitzner-Allee 6, 85540 Haar, Germany                 */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Author      : SOFTING                                                    */
/*                                                                           */
/*  CHANGE_NOTES                                                             */
/*                                                                           */
/*  date      name      change                                               */
/*  -----------------------------------------------------------------------  */
/*****************************************************************************/
/*****************************************************************************/

#ifndef _XX_IF_H_
#define _XX_IF_H_

/* --- OK return value ----------------------------------------------------- */
#define E_OK                0        /* no errors                            */

/* --- contants for IRQ values --------------------------------------------- */
#define NULL_IRQ                 0x00         /* NULL IRQ (for shared IRQ's) */
#define REQ_IRQ                  0xF0             /* IND/CON IRQ is received */
#define ACK_IRQ                  0x0F                 /* acknowledge REQ/RES */
#define DP_SLAVE_IO_REQ_IRQ      0xE0                 /* DP-SLAVE-IO IND/REQ */
#define DP_SLAVE_IO_ACK_IRQ      0x0E     /* DP-SLAVE-IO acknowldege IND/REQ */
#define DP_DATA_STOP_REQ_IRQ     0xD0     /* data transfer stop REQ received */
#define DP_DATA_STOP_ACK_IRQ     0x0D      /* data transfer stop Acknowldege */
#define TRC_REQ_IRQ              0xC0               /* TRACE REQ is received */
#define TRC_ACK_IRQ              0x0C                   /* TRACE acknowledge */
#define EXCEPTION_REQ_IRQ        0xB0                   /* Exception Request */
#define EXCEPTION_ACK_IRQ        0x0B               /* Exception Acknowledge */
#define DPS_SLAVE_INP_REQ_IRQ    0xA0       /* SLAVE: DP-SLAVE-INPUT request */
#define DPS_SLAVE_INP_ACK_IRQ    0x0A   /* SLAVE: DP-SLAVE-INPUT Acknowldege */
#define DPS_SLAVE_OUTP_REQ_IRQ   0x90      /* SLAVE: DP-SLAVE-OUTPUT request */
#define DPS_SLAVE_OUTP_ACK_IRQ   0x09  /* SLAVE: DP-SLAVE-OUTPUT Acknowldege */

/* --- contants to receive an indication or confirmation ------------------- */
#define CON_IND_RECEIVED    0x0001  /* ind. or con. has been received        */
#define NO_CON_IND_RECEIVED 0x0000  /* nothing has been received             */

#define REQ                 0x00   /* request primitive                      */
#define CON                 0x01   /* confirmation primitive                 */
#define IND                 0x02   /* indication primitive                   */
#define RES                 0x03   /* response primitive                     */


#ifdef PB
/* --- layer identifiers --------------------------------------------------- */
#define USR                 0x00      /* identifier USER                     */
#define FMS                 0x01      /* identifier FMS                      */
#define LLI                 0x02      /* identifier LLI                      */
#define FDL                 0x03      /* identifier FDL                      */
#define FM7                 0x04      /* identifier FM7                      */
#define FMB                 0x05      /* identifier FMB                      */
#define FMS_USR             0x06      /* identifier FMS-USER                 */
#define FM7_USR             0x07      /* identifier FM7-USER                 */
#define FMB_USR             0x08      /* identifier FMB-User                 */
#define DPS                 0x09      /* identifier DPS */
#define DPS_USR             0x0A      /* identifier DPS-USER */
#define SM7                 0x09      /* identifier SM7                      */
#define SM7_USR             0x0A      /* identifier SM7_USR                  */
#define DP                  0x0B      /* identifier DP                       */
#define DP_USR              0x0C      /* identifier DP-USER                  */
#define FDLIF               0x0D      /* identifier FDLIF                    */
#define FDLIF_USR           0x0E      /* identifier FDLIF-User               */

#define PAPI                0xF0      /* identifier Application-Layer-IF     */

/* --- identifiers for data description interface -------------------------- */
/* --- 0x00 - 0x3F used by FMS -- (no identifiers defined) ----------------- */

/* --- 0x40 - 0x7F reserved by management -- (no identifiers defined) ------ */

/* --- 0x80 - 0x9F reserved by DP ------------------------------------------ */
#define ID_DP_SLAVE_IO_IMAGE         0x80        /* image for slave io datas */
#define ID_DP_STATUS_IMAGE           0x81          /* image for status datas */
#define ID_DP_SLAVE_IO_CONFIG_IMAGE  0x82        /* DPRAM layout of slave io */

#define ID_DPS_SLAVE_INPUT_IMAGE     0x90  /* SLAVE: image for slave input datas */
#define ID_DPS_SLAVE_OUTPUT_IMAGE    0x91 /* SLAVE: image for slave output datas */

/* --- 0xA0 - 0xEF reserved for future use -- (no identifiers defined) ----- */

/* --- 0xF0 - 0xFF reserved for diagonsis  --------------------------------- */
#define ID_EXCEPTION_IMAGE           0xF0 /* image for exeption datas        */
#define ID_FW_VERS_IMAGE             0xF1 /* image for firmware version      */
#define ID_SERIAL_DEVICE_NUMBER      0xf2

/* --- idebtifier for end of data description list ------------------------- */
#define EOF_DATA_DESCR_LIST          0x00 /* end of data description list    */

#endif // PB


#ifdef FF
/* --- layer identifiers --------------------------------------------------- */
#define USR                 0x01      /* identifier USER                     */
#define FMS                 0x02      /* identifier FMS                      */
#define FAS                 0x03      /* identifier FAS                      */
#define DLL                 0x04      /* identifier DLL                      */
#define NMA                 0x05      /* identifier NMA                      */
#define SM                  0x06      /* identifier SM                       */
#define FMS_USR             0x07      /* identifier FMS-USER                 */
#define NMA_USR             0x08      /* identifier NMA-USER                 */
#define SM_USR              0x09      /* identifier SM-USER                  */

/* --- identifiers for data description interface -------------------------- */
/* --- 0x00 - 0x3F used by FMS -- (no identifiers defined) ----------------- */

/* --- 0x40 - 0x9F reserved by management -- (no identifiers defined) ------ */

/* --- 0xA0 - 0xEF reserved for future use -- (no identifiers defined) ----- */

/* --- 0xF0 - 0xFF reserved for diagnosis  --------------------------------- */
#define ID_EXCEPTION_IMAGE           0xF0 /* image for exeption datas        */
#define ID_FW_VERS_IMAGE          0xF1 /* image for firmware version      */

/* --- idebtifier for end of data description list ------------------------- */
#define EOF_DATA_DESCR_LIST          0x00 /* end of data description list    */
#endif //FF


#ifdef PB
/*****************************************************************************/
/* PROFI SERVICE DESCRIPTION BLOCK                                           */
/*****************************************************************************/

typedef struct _T_PROFI_SERVICE_DESCR
{
  USIGN16     comm_ref;                /* communication reference            */
  USIGN8      layer;                   /* layer                              */
  USIGN8      service;                 /* service identifier                 */
  USIGN8      primitive;               /* service primitive                  */
  INT8        invoke_id;               /* invoke id                          */
  INT16       result;                  /* service result (POS or NEG)        */
#ifdef WIN32
} T_PROFI_SERVICE_DESCR;
#else
} __attribute__ ((packed)) T_PROFI_SERVICE_DESCR;
#endif
#endif //PB



#ifdef FF
/*****************************************************************************/
/* FIELDBUS SERVICE DESCRIPTION BLOCK                                        */
/*****************************************************************************/

typedef struct _T_FIELDBUS_SERVICE_DESCR
{
  USIGN16     comm_ref;                /* communication reference            */
  USIGN8      layer;                   /* layer                              */
  USIGN8      service;                 /* service identifier                 */
  USIGN8      primitive;               /* service primitive                  */
  INT8        invoke_id;               /* invoke id                          */
  INT16       result;                  /* service result (POS or NEG)        */
#ifdef WIN32
} T_FIELDBUS_SERVICE_DESCR;
#else
} __attribute__ ((packed)) T_FIELDBUS_SERVICE_DESCR;
#endif
#endif // FF


#endif
