/******************************************************************************
*                                                                             *
*                                 SOFTING AG                                  *
*                          Richard-Reitzner-Allee 6                           *
*                                D-85540 Haar                                 *
*                        Phone: (++49)-(0)89-45656-0                          *
*                        Fax:   (++49)-(0)89-45656-399                        *
*                                                                             *
*                      Copyright (C) SOFTING AG 1995-2003                     *
*                              All Rights Reserved                            *
*                                                                             *
*******************************************************************************

FILE_NAME               DPDEMO.C

PROJECT_NAME            PROFIBUS

MODULE                  DPDEMO

COMPONENT_LIBRARY       --------

AUTHOR                  SOFTING AG

VERSION                 5.24

DATE                    31.07.2001

STATUS                  finished

FUNCTIONAL_MODULE_DESCRIPTION

DP Demo application


RELATED_DOCUMENTS

============================================================================*/
#include "keywords.h"

INCLUDES

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if defined (WIN16) || defined (WIN32)
#include <conio.h>
#include <windows.h>
#endif

#include "pb_type.h"
#include "pb_conf.h"
#include "pb_if.h"
#include "pb_err.h"
#include "pb_fmb.h"
#include "pb_dp.h"

/****************************************************************************/

LOCAL_DEFINES

#if defined (WIN32) || defined (_LINUX)
#define BOARD_NUMBER            0
#else
#define H_DPR_BASE_ADDRESS      0xD0000000uL
#define IO_ADDRESS              0x0240
#endif

#define DP_MAX_SERVICE_RETRY    10

#define OUTPUT_STRING(x)        puts (x)

#define SWAP16(x)               ( (x >> 8) | (x << 8) )

#define PROFI_RCV_CON_IND()     do                                                                                 \
                                {                                                                                  \
                                  con_ind_buffer_len = 256;                                                        \
                                  result = profi_rcv_con_ind (&con_ind_sdb, con_ind_buffer, &con_ind_buffer_len); \
                                  if (kbhit ()) if (getch () == 'q') exit (0);                                     \
                                } while (result == NO_CON_IND_RECEIVED);                                           \
                                                                                                                   \
                                if (result == E_IF_FATAL_ERROR) handle_fatal_error ((T_EXCEPTION*) con_ind_buffer)


/****************************************************************************/

LOCAL_TYPEDEFS

typedef struct _T_DP_IO_DESCR
{
  USIGN8    number_inputs;
  USIGN8    number_outputs;

  USIGN16   offset_inputs;
  USIGN16   offset_outputs;

} T_DP_IO_DESCR;

/****************************************************************************/

FUNCTION_DECLARATIONS
static VOID print_version(VOID);
static VOID print_papi_firmware_version(VOID);
static VOID check_options (char options [] );



/****************************************************************************/

EXPORT_DATA

/****************************************************************************/

IMPORT_DATA

/****************************************************************************/

LOCAL_DATA

#if defined (WIN16) || defined (WIN32)
static char                    exe_fname [_MAX_FNAME];  /* name of .exe file */
#endif

static PB_BOOL                 load_firmware;

#if defined (WIN32) || defined (_LINUX)
static USIGN16                 board_number;
#else
static USIGN32                 dpr_base_address;
static USIGN16                 io_address;
#endif

static char                     s [128];
static PB_BOOL                  prg_done = PB_FALSE;

static INT16                    result;

static USIGN8                   con_ind_buffer [256];
static USIGN16                  con_ind_buffer_len;
static T_PROFI_SERVICE_DESCR    con_ind_sdb;

static PB_BOOL                  slave_diag_requested = PB_FALSE;
static PB_BOOL                  parallel_service     = PB_FALSE;

static T_DP_IO_DESCR            io_descr [DP_MAX_NUMBER_SLAVES];
USIGN8                          test_data [DP_MAX_TELEGRAM_LEN];


/****************************************************************************/

#include "dpdemo.h"

/****************************************************************************/
/****************************************************************************/
/****************************************************************************/

FUNCTION LOCAL VOID handle_fatal_error
  (
    IN T_EXCEPTION * exception
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* This function is used to handle a fatal error                          */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - NONE                                                               */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

  sprintf (s, "task_id: 0x%02hX", exception->task_id);
  OUTPUT_STRING (s);

  sprintf (s, "par_1  : 0x%02hX", exception->par1);
  OUTPUT_STRING (s);

  sprintf (s, "par_2  : 0x%04X", exception->par2);
  OUTPUT_STRING (s);

  sprintf (s, "par_3  : 0x%04X", exception->par3);
  OUTPUT_STRING (s);

  exit (0);

} /* handle_fatal_error */

/****************************************************************************/

FUNCTION LOCAL PB_BOOL fmb_set_configuration_req
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*                                                                        */
  /* The function is used to configure the protocol stack running in the    */
  /* firmware. The service sets the active components and allocates memory  */
  /* for the number of devices specified.                                   */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - PB_TRUE                                                            */
  /*   - PB_FALSE                                                           */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  T_FMB_SET_CONFIGURATION_REQ   set_config_req;
  T_PROFI_SERVICE_DESCR         sdb;

  T_FMB_CONFIG_DP               FAR *config_dp_ptr;


FUNCTION_BODY

  sdb.layer     = FMB;
  sdb.service   = FMB_SET_CONFIGURATION;
  sdb.primitive = REQ;

  config_dp_ptr = (T_FMB_CONFIG_DP FAR*) &(set_config_req.dp);

  set_config_req.fms_active         = PB_FALSE;
  set_config_req.dp_active          = PB_TRUE;
  set_config_req.fdlif_active       = PB_FALSE;
  set_config_req.fdl_evt_receiver   = DP_USR;
  /* FDL events are reproted to the application using DP */
  set_config_req.sm7_active         = PB_FALSE;
  set_config_req.data_buffer_length = 256;

  config_dp_ptr->max_number_slaves      = 32;
  config_dp_ptr->max_slave_output_len   = 32;
  config_dp_ptr->max_slave_input_len    = 32;
  config_dp_ptr->max_slave_diag_entries = 64;
  config_dp_ptr->max_slave_diag_len     = 32;
  config_dp_ptr->max_bus_para_len       = 128;
  config_dp_ptr->max_slave_para_len     = 128;

  sprintf (s, "FMB_Set_Configuration.req [%d]", profi_snd_req_res (&sdb, &set_config_req, PB_FALSE));
  OUTPUT_STRING (s);

  PROFI_RCV_CON_IND (); /*Macro to fetch one confirmation or indication*/

  if ( (con_ind_sdb.service   == FMB_SET_CONFIGURATION) &&
       (con_ind_sdb.primitive == CON                  ) &&
       (con_ind_sdb.result    == POS                  ) )
  {
    sprintf (s, "FMB_Set_Configuration.con [+]");
    OUTPUT_STRING (s);

    return (PB_TRUE);
  }

  sprintf (s, "FMB_Set_Configuration.con [-]");
  OUTPUT_STRING (s);

  return (PB_FALSE);
} /* fmb_set_configuration_req */

/****************************************************************************/

FUNCTION LOCAL PB_BOOL dp_init_master_req
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /* This function is used to define the DP-Master operating mode           */
  /* (e.g. class1/class2 master operation)                                  */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - PB_TRUE                                                            */
  /*   - PB_FALSE                                                           */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  T_DP_INIT_MASTER_REQ    init_master_req;
  T_PROFI_SERVICE_DESCR   sdb;


FUNCTION_BODY

  sdb.layer     = DP;
  sdb.service   = DP_INIT_MASTER;
  sdb.primitive = REQ;

  init_master_req.master_default_address = DP_MASTER_ADDRESS;
  init_master_req.master_class2          = PB_FALSE;                      /* master class 1 operation */
  init_master_req.lowest_slave_address   = 0;
#if defined (WINNT) || defined (_LINUX)
  init_master_req.slave_io_address_mode  = DP_AAM_IO_BLOCKS;     /* only mode possible with NT driver */
#else
  init_master_req.slave_io_address_mode  = DP_AAM_ARRAY;     /* same amount of memory for every slave */
#endif
  init_master_req.clear_outputs          = PB_FALSE;  /* set outputs to 'clear' befor starting to poll*/
  init_master_req.auto_remote_services   = DP_AUTO_REMOTE_SERVICES; /* remote .ind handled by firmware*/
  init_master_req.cyclic_data_transfer   = PB_TRUE;

  sprintf (s, "DP_Init_Master.req [%d]", profi_snd_req_res (&sdb, &init_master_req, PB_FALSE));
  OUTPUT_STRING (s);

  PROFI_RCV_CON_IND (); /* macro to fetch a confirmation or indication */

  if ( (con_ind_sdb.service   == DP_INIT_MASTER) &&
       (con_ind_sdb.primitive == CON           ) &&
       (con_ind_sdb.result    == POS           ) )
  {
    sprintf (s, "DP_Init_Master.con [+]");
    OUTPUT_STRING (s);

    return (PB_TRUE);
  }

  sprintf (s, "DP_Init_Master.con [-]");
  OUTPUT_STRING (s);

  return (PB_FALSE);
} /* dp_init_master_req */

/****************************************************************************/

FUNCTION LOCAL PB_BOOL dp_download_bus_req
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* Loads the busparameter set to the protocol stack                       */
  /*                                                                        */
  /* NOTE: If you're operating multiple protocols (FMS/DP) simultaneously   */
  /*       service fmb_set_busparameter must be used instead                */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - PB_TRUE                                                            */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  USIGN8                  download_buffer [244];
  T_PROFI_SERVICE_DESCR   sdb;

  T_DP_DOWNLOAD_REQ       FAR *download_req_ptr;
  T_DP_BUS_PARA_SET       FAR *bus_para_set_ptr;


FUNCTION_BODY

  sdb.layer     = DP;
  sdb.service   = DP_DOWNLOAD_LOC;
  sdb.primitive = REQ;

  bus_para_set_ptr = (T_DP_BUS_PARA_SET FAR*) dp_bus_para_set;
  download_req_ptr = (T_DP_DOWNLOAD_REQ FAR*) download_buffer;

  download_req_ptr->data_len   = SWAP16 (bus_para_set_ptr->bus_para_len);
  download_req_ptr->rem_add    = 0;
  download_req_ptr->area_code  = DP_AREA_BUS_PARAM;
  download_req_ptr->add_offset = 0;

  memcpy (download_req_ptr + 1, bus_para_set_ptr, download_req_ptr->data_len);

  sprintf (s, "DP_Download_Loc.req, area_code = %hu [%d]", download_req_ptr->area_code, profi_snd_req_res (&sdb, download_req_ptr, PB_FALSE));
  OUTPUT_STRING (s);

  PROFI_RCV_CON_IND ();

  if ( (con_ind_sdb.service   == DP_DOWNLOAD_LOC) &&
       (con_ind_sdb.primitive == CON            ) &&
       (con_ind_sdb.result    == POS            ) )
  {
    sprintf (s, "DP_Download_Loc.con [+]: status = 0x%04X", ((T_DP_DOWNLOAD_RES_CON FAR*) con_ind_buffer)->status);
    OUTPUT_STRING (s);
  }
  else
  {
    sprintf (s, "DP_Download_Loc.con [-]: status = 0x%04X", ((T_DP_DOWNLOAD_RES_CON FAR*) con_ind_buffer)->status);
    OUTPUT_STRING (s);
  }

  return (PB_TRUE);
} /* dp_download_bus_req */

/****************************************************************************/

FUNCTION LOCAL PB_BOOL dp_download_slave_req
  (
  IN    USIGN8    slave_offset
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* Loads one slave parameter set to the protocol stack.                   */
  /* The parameter sets are defined in DPDEMO.H. For exact meaning of each  */
  /* individual parameter refer to the PROFIBUS standard.                   */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - PB_TRUE                                                            */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  USIGN8                  download_buffer [244];
  T_PROFI_SERVICE_DESCR   sdb;

  T_DP_DOWNLOAD_REQ       FAR *download_req_ptr;
  T_DP_SLAVE_PARA_SET     FAR *slave_para_set_ptr;


FUNCTION_BODY

  sdb.layer     = DP;
  sdb.service   = DP_DOWNLOAD_LOC;
  sdb.primitive = REQ;

  slave_para_set_ptr = (T_DP_SLAVE_PARA_SET FAR*) slave_para_table [dp_slave_table [slave_offset]];
  download_req_ptr   = (T_DP_DOWNLOAD_REQ FAR*) download_buffer;

  download_req_ptr->data_len   = SWAP16 (slave_para_set_ptr->slave_para_len);
  download_req_ptr->rem_add    = 0;
  download_req_ptr->area_code  = dp_slave_table [slave_offset];
  download_req_ptr->add_offset = 0;

  memcpy (download_req_ptr + 1, slave_para_set_ptr, download_req_ptr->data_len);

  sprintf (s, "DP_Download_Loc.req, area_code = %hu [%d]", download_req_ptr->area_code, profi_snd_req_res (&sdb, download_req_ptr, PB_FALSE));
  OUTPUT_STRING (s);

  PROFI_RCV_CON_IND ();

  if ( (con_ind_sdb.service   == DP_DOWNLOAD_LOC) &&
       (con_ind_sdb.primitive == CON            ) &&
       (con_ind_sdb.result    == POS            ) )
  {
    sprintf (s, "DP_Download_Loc.con [+]: status = 0x%04X", ((T_DP_DOWNLOAD_RES_CON FAR*) con_ind_buffer)->status);
    OUTPUT_STRING (s);
  }
  else
  {
    sprintf (s, "DP_Download_Loc.con [-]: status = 0x%04X", ((T_DP_DOWNLOAD_RES_CON FAR*) con_ind_buffer)->status);
    OUTPUT_STRING (s);
  }

  return (PB_TRUE);
} /* dp_download_slave_req */

/****************************************************************************/

FUNCTION LOCAL PB_BOOL dp_act_param_loc_req
  (
  IN    USIGN8    identifier
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /* Controls the operating mode of the DP protocol stack. The service is   */
  /* used to assign a particular mode of operation to the stack (OFFLINE,   */
  /* STOP, CLEAR, OPERATE - according to the PROFIBUS standard).            */
  /*                                                                        */
  /* Furthermore, it may be used to activate the busparameter set downloaded*/
  /* and to (de-)activate individual slaves.                                */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - 0                                                                  */
  /*   - 1                                                                  */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  T_DP_ACT_PARAM_REQ      act_param_req;
  T_PROFI_SERVICE_DESCR   sdb;


FUNCTION_BODY

  sdb.layer     = DP;
  sdb.service   = DP_ACT_PARAM_LOC;
  sdb.primitive = REQ;

  act_param_req.rem_add   = 0;
  act_param_req.area_code = DP_AREA_SET_MODE;
  act_param_req.activate  = identifier;

  switch (identifier)
  {
    case DP_OP_MODE_STOP   : sprintf (s, "DP_Act_Param_Loc.req, change to STOP");    break;
    case DP_OP_MODE_CLEAR  : sprintf (s, "DP_Act_Param_Loc.req, change to CLEAR");   break;
    case DP_OP_MODE_OPERATE: sprintf (s, "DP_Act_Param_Loc.req, change to OPERATE"); break;
  }

  OUTPUT_STRING (s);

  result = profi_snd_req_res (&sdb, &act_param_req, PB_FALSE);

  return ((PB_BOOL) (result == E_OK));
} /* dp_act_param_loc_req */

/****************************************************************************/

FUNCTION LOCAL PB_BOOL dp_get_slave_diag_req
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /* Fetch one diagnostic message from diagnostic message buffer            */
  /* of the stack                                                           */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - 0                                                                  */
  /*   - 1                                                                  */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR   sdb;


FUNCTION_BODY

  sdb.layer     = DP;
  sdb.service   = DP_GET_SLAVE_DIAG;
  sdb.primitive = REQ;

  result = profi_snd_req_res (&sdb, &sdb, PB_FALSE);

  sprintf (s, "DP_Get_Slave_Diag.req [%d]", result);
  OUTPUT_STRING (s);

  return ((PB_BOOL) (result == E_OK));
} /* dp_get_slave_diag_req */

/****************************************************************************/

FUNCTION LOCAL VOID dp_get_slave_diag_con
  (
  IN    T_DP_GET_SLAVE_DIAG_CON   FAR* get_slave_diag_con_ptr
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /* This function is used to decode the diagnostic information received    */
  /* by the protocol stack                                                  */
  /*                                                                        */
  /* possible return values:                                                */
  /*   NONE                                                                 */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  T_DP_DIAG_DATA    FAR *diag_data_ptr;


FUNCTION_BODY

  if (get_slave_diag_con_ptr->diag_data_len >= DP_MIN_SLAVE_DIAG_LEN)
  {
    diag_data_ptr = (T_DP_DIAG_DATA FAR*) (get_slave_diag_con_ptr + 1);

    sprintf (s, "Slave [%3hu] [0x%04hX]: Status = 0x%02hX 0x%02hX 0x%02hX, Master = %3hu, Ext = %hu, Diags = %hu",
             get_slave_diag_con_ptr->rem_add,
             SWAP16 (diag_data_ptr->ident_number),
             diag_data_ptr->station_status_1,
             diag_data_ptr->station_status_2,
             diag_data_ptr->station_status_3,
             diag_data_ptr->master_add,
             get_slave_diag_con_ptr->diag_data_len - DP_MIN_SLAVE_DIAG_LEN,
             get_slave_diag_con_ptr->diag_entries);
    OUTPUT_STRING (s);

  } /* diag_data_len */
} /* dp_get_slave_diag_con */

/****************************************************************************/

FUNCTION LOCAL PB_BOOL dp_io_offsets
  (
  IN  VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /* calculate the offsets of inputs and outputs. To exchange data in DP,   */
  /* I/O information has to be written to particular offsets for every      */
  /* slave. This function returns the offsets for a particular slave.       */
  /*                                                                        */
  /* possible return values:                                                */
  /*   - PB_FALSE                                                           */
  /*   - PB_TRUE                                                            */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR         sdb;
  T_DP_GET_SLAVE_PARAM_REQ      get_slave_param_req;
  T_DP_GET_SLAVE_PARAM_CON      FAR *get_slave_param_con_ptr;
  T_DP_SLAVE_PARAM_SLAVE_INFO   FAR *slave_info_ptr;

  USIGN8                        slave;


FUNCTION_BODY

  memset (io_descr, 0, sizeof (T_DP_IO_DESCR) * DP_MAX_NUMBER_SLAVES);

  /*------------------------------------------------------------------------*/

  for (slave = 0; dp_slave_table [slave] != 0xFF; slave++)
  {
    sdb.layer     = DP;
    sdb.service   = DP_GET_SLAVE_PARAM;
    sdb.primitive = REQ;

    get_slave_param_req.identifier = DP_SLAVE_PARAM_SLAVE_INFO;
    get_slave_param_req.rem_add    = dp_slave_table [slave];

    result = profi_snd_req_res (&sdb, &get_slave_param_req, PB_FALSE);

    sprintf (s, "DP_Get_Slave_Param.req [%d]", result);
    OUTPUT_STRING (s);

    if (result != E_OK) return (PB_FALSE);

    /*----------------------------------------------------------------------*/

    PROFI_RCV_CON_IND ();

    get_slave_param_con_ptr = (T_DP_GET_SLAVE_PARAM_CON FAR*) con_ind_buffer;

    if ( (con_ind_sdb.service   == DP_GET_SLAVE_PARAM) &&
         (con_ind_sdb.primitive == CON               ) &&
         (con_ind_sdb.result    == POS               ) )
    {
      slave_info_ptr = (T_DP_SLAVE_PARAM_SLAVE_INFO FAR*) (get_slave_param_con_ptr + 1);

      sprintf (s, "DP_Get_Slave_Param.con [+] [%3hu]: Inputs [%04X] = %3u, Outputs [%04X] = %3u",
               dp_slave_table [slave],
               slave_info_ptr->offset_inputs,
               slave_info_ptr->number_inputs,
               slave_info_ptr->offset_outputs,
               slave_info_ptr->number_outputs
              );
      OUTPUT_STRING (s);

      io_descr [dp_slave_table [slave]].number_inputs  = slave_info_ptr->number_inputs;
      io_descr [dp_slave_table [slave]].number_outputs = slave_info_ptr->number_outputs;
      io_descr [dp_slave_table [slave]].offset_inputs  = slave_info_ptr->offset_inputs;
      io_descr [dp_slave_table [slave]].offset_outputs = slave_info_ptr->offset_outputs;
    }
    else
    {
      sprintf (s, "DP_Get_Slave_Param.con [-]: status = 0x%04X", get_slave_param_con_ptr->status);
      OUTPUT_STRING (s);

      return (PB_FALSE);
    }
  } /* for slave */

  return (PB_TRUE);

} /* dp_io_offsets */

/****************************************************************************/

FUNCTION LOCAL VOID dp_data_exchange
  (
  IN  VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /* This function is used to exchange data with the slaves                 */
  /*                                                                        */
  /* possible return values:                                                */
  /*   NONE                                                                 */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  USIGN8              data [DP_MAX_TELEGRAM_LEN];
  USIGN16             data_len;

  USIGN8              slave;
  static int          zaehler = 0;

  INT16               result;              /* !!! local result variable !!! */
  USIGN16             retry_counter;

  USIGN8              slaveaddress;
  USIGN16              dpramoffset;
  int i;

FUNCTION_BODY

  for (slave = 0; dp_slave_table [slave] != 0xFF; slave++)
  {
    data_len = io_descr [dp_slave_table [slave]].number_inputs;
    slaveaddress = dp_slave_table [slave];
    dpramoffset = io_descr [dp_slave_table [slave]].offset_inputs;

    printf("reading %u input bytes from slave %u at offset [%04X]", data_len, slaveaddress, dpramoffset);
    /*--- GET CURRENT OUTPUT STATE -----------------------------------------*/

    retry_counter = DP_MAX_SERVICE_RETRY;

    do
    {
      
 
      
      result = profi_get_data
      (
        ID_DP_SLAVE_IO_IMAGE,
        io_descr [dp_slave_table [slave]].offset_inputs,
        &data_len,
        data
      );


 
      for(i=0; i<data_len; i++)
      {
         
         if(!(i%16))
           printf("\n0x%08x: ",i);
         printf("0x%02x ", data[i]);
      }
      printf("\n\n");
      

    } while ((result != E_OK) && (retry_counter-- > 0));


    /*--- SET NEW OUTPUT VALUES --------------------------------------------*/
    data_len = io_descr [dp_slave_table [slave]].number_outputs;
    retry_counter = DP_MAX_SERVICE_RETRY;
    dpramoffset = io_descr [dp_slave_table [slave]].offset_outputs;

    if (zaehler < 100) 
    {
      test_data[0] = slave;
      test_data[1] = zaehler;
      zaehler++;
    }
    if (zaehler == 100) 
    {
      zaehler = 0;
      test_data[0] = slave;
      test_data[1] = zaehler;
    }

      printf("writing %u output bytes to slave %u at offset [%04X]", data_len, slaveaddress, dpramoffset);

      for(i=0; i<data_len; i++)
      {
         
         if(!(i%16)) printf("\n0x%08x: ",i);
         printf("0x%02x ", test_data[i]);
      }
      printf("\n\n");

    do
    {
     //(ID_DP_SLAVE_IO_IMAGE)
     // printf("len=%u, offset=0x%x\n", data_len, 
 //io_descr[dp_slave_table [slave]].offset_outputs);


      result = profi_set_data
      (
        ID_DP_SLAVE_IO_IMAGE,
        io_descr [dp_slave_table [slave]].offset_outputs,
        data_len,
        test_data
      );

    } while ((result != E_OK) && (retry_counter-- > 0));

  } /* for slave */

} /* dp_data_exchange */

/****************************************************************************/

FUNCTION LOCAL VOID handle_communication
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*   This function is used to handle the communication                    */
  /*                                                                        */
  /* possible return values:                                                */
  /*   NONE                                                                 */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES


  PB_BOOL                   init_result;
  USIGN8                    slave;
  USIGN8                    usif_state = DP_OP_MODE_OFFLINE;


  T_DP_GET_SLAVE_DIAG_CON   FAR *get_slave_diag_con_ptr;


FUNCTION_BODY

  /*------------------------------------------------------------------------*/
  /*--- STARTUP - NO EVENTS ------------------------------------------------*/
  /*------------------------------------------------------------------------*/
  printf("\n\nDownloading parameter sets to PROFIBUS interface...\n\n");
  init_result  = fmb_set_configuration_req ();
  init_result &= dp_init_master_req ();
  init_result &= dp_download_bus_req ();

  for (slave = 0; dp_slave_table [slave] != 0xFF; slave++)
  {
    init_result &= dp_download_slave_req (slave);
  }

  init_result &= dp_io_offsets ();
  init_result &= dp_act_param_loc_req (DP_OP_MODE_STOP);


  printf("\n\nExecution paused...\n Press 'q' to quit\n press return to continue\n\n");

  while(!kbhit());
  if( getch()=='q') return;

  /*------------------------------------------------------------------------*/
  /*--- CYCLIC EVENT HANDLING ----------------------------------------------*/
  /*------------------------------------------------------------------------*/
  /* During data_exchange, the application must be able to cope with any    */
  /* type of received information. This is why all possible conditions are  */
  /* compiled in one case-structure                                         */
  /*                                                                        */
  /**************************************************************************/
  printf("\n\nStarting communication...\n\n");

  if (init_result)
  {
    do
    {
       do
        {
          con_ind_buffer_len = 256;
          result = profi_rcv_con_ind (&con_ind_sdb, con_ind_buffer, &con_ind_buffer_len);

          if (kbhit ()) if (getch () == 'q') exit (0);
          if (result == NO_CON_IND_RECEIVED) dp_data_exchange ();
        } while (result == NO_CON_IND_RECEIVED);

        if (result == E_IF_FATAL_ERROR) handle_fatal_error ((T_EXCEPTION*)con_ind_buffer);

      if (con_ind_sdb.primitive == CON)
      {
        if (con_ind_sdb.result == POS)
        {
          switch (con_ind_sdb.service)
          {
            /*--------------------------------------------------------------*/

            case DP_ACT_PARAM_LOC:
            {
              sprintf (s, "DP_Act_Param_Loc.con [+]");
              OUTPUT_STRING (s);

              if (usif_state == DP_OP_MODE_OFFLINE)
              {
                usif_state = DP_OP_MODE_STOP;

                dp_act_param_loc_req (DP_OP_MODE_CLEAR);
              }
              else if (usif_state == DP_OP_MODE_STOP)
              {
                usif_state = DP_OP_MODE_CLEAR;

                dp_act_param_loc_req (DP_OP_MODE_OPERATE);
              }
              else if (usif_state == DP_OP_MODE_CLEAR)
              {
                usif_state = DP_OP_MODE_OPERATE;

                if ( (parallel_service) && (! slave_diag_requested) )
                {
                  if (dp_get_slave_diag_req ())
                  {
                    slave_diag_requested = PB_TRUE;
                    parallel_service     = PB_FALSE;
                  }
                }
              }

              break;
            } /* case DP_ACT_PARAM_LOC */

            /*--------------------------------------------------------------*/

            case DP_GET_SLAVE_DIAG:
            {
              get_slave_diag_con_ptr = (T_DP_GET_SLAVE_DIAG_CON FAR*) con_ind_buffer;

              sprintf (s, "DP_Get_Slave_Diag.con [+]");
              OUTPUT_STRING (s);

              dp_get_slave_diag_con (get_slave_diag_con_ptr);

              slave_diag_requested = PB_FALSE;

              if (get_slave_diag_con_ptr->diag_entries)
              {
                slave_diag_requested = PB_TRUE;
                dp_get_slave_diag_req ();
              }
              break;
            } /* case DP_GET_SLAVE_DIAG */

            /*--------------------------------------------------------------*/

            default:
            {
              sprintf (s, "CON [+], service = %hu", con_ind_sdb.service);
              OUTPUT_STRING (s);

            } /* deafult service */
          } /* switch */
        } /* if POS */
        else
        {
          sprintf (s, "negative confirmation received, aborting");
          OUTPUT_STRING (s);

          prg_done = PB_TRUE;

        } /* else POS */
      } /* if CON */
      else if (con_ind_sdb.primitive == IND)
      {
        if (con_ind_sdb.result == POS)
        {
          switch (con_ind_sdb.service)
          {
            case FMB_FM2_EVENT:
            {
              switch (((T_FMB_FM2_EVENT_IND FAR*) con_ind_buffer)->reason)
              {
                case FM2_FAULT_ADDRESS     : sprintf (s, "FMB.ind: duplicate address recognized"); break;
                case FM2_FAULT_PHY         : sprintf (s, "FMB.ind: phys.layer is malfunctioning"); break;
                case FM2_FAULT_TTO         : sprintf (s, "FMB.ind: time out on bus detected    "); break;
                case FM2_FAULT_SYN         : sprintf (s, "FMB.ind: no receiver synchronization "); break;
                case FM2_FAULT_OUT_OF_RING : sprintf (s, "FMB.ind: station out of ring         "); break;
                case FM2_GAP_EVENT         : sprintf (s, "FMB.ind: new station in ring         "); break;

                default                    : sprintf (s, "FMB.ind: unknown reason code received");

              } /* switch reason */

              OUTPUT_STRING (s);
              break;
            } /* case FMB_FM2_EVENT */

            /*--------------------------------------------------------------*/

            case DP_ACT_PARAM_LOC:
            {
              usif_state = ((T_DP_ACT_PARAM_IND FAR*) con_ind_buffer)->activate;

              switch (usif_state)
              {
                case DP_OP_MODE_STOP   : sprintf (s, "DP_Act_Param_Loc.ind, mode changed to STOP");    break;
                case DP_OP_MODE_CLEAR  : sprintf (s, "DP_Act_Param_Loc.ind, mode changed to CLEAR");   break;
                case DP_OP_MODE_OPERATE: sprintf (s, "DP_Act_Param_Loc.ind, mode changed to OPERATE"); break;
              }

              OUTPUT_STRING (s);

              if (usif_state == DP_OP_MODE_STOP)
              {
                usif_state = DP_OP_MODE_CLEAR;

                dp_act_param_loc_req (DP_OP_MODE_CLEAR);
              }
              else if (usif_state == DP_OP_MODE_CLEAR)
              {
                usif_state = DP_OP_MODE_OPERATE;

                dp_act_param_loc_req (DP_OP_MODE_OPERATE);
              }

              break;
            } /* case DP_ACT_PARAM_LOC */

            /*--------------------------------------------------------------*/

            case DP_GET_SLAVE_DIAG:
            {
              get_slave_diag_con_ptr = (T_DP_GET_SLAVE_DIAG_CON FAR*) con_ind_buffer;

              sprintf (s, "DP_Get_Slave_Diag.ind");
              OUTPUT_STRING (s);

              dp_get_slave_diag_con (get_slave_diag_con_ptr);

              if ( (get_slave_diag_con_ptr->diag_entries) &&
                   (! slave_diag_requested              ) )
              {
                if (dp_get_slave_diag_req ())
                {
                  slave_diag_requested = PB_TRUE;
                }
                else
                {
                  parallel_service = PB_TRUE;
                }
              }

              break;
            } /* case DP_GET_SLAVE_DIAG */

            /*--------------------------------------------------------------*/

            default:
            {
              sprintf (s, "IND [+], service = %hu", con_ind_sdb.service);
              OUTPUT_STRING (s);

            } /* deafult service */
          } /* switch */
        } /* if POS */
        else
        {
          sprintf (s, "negative indication received, aborting");
          OUTPUT_STRING (s);

          prg_done = PB_TRUE;

        } /* else POS */
      } /* if IND */
      else
      {
        sprintf (s, "unknown primitve received, aborting");
        OUTPUT_STRING (s);

        prg_done = PB_TRUE;
      }


    } while (! prg_done);
  } /* if startup ok */
} /* handle_communication */

/****************************************************************************/

FUNCTION LOCAL VOID exit_communication
  (
  IN    VOID
  )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*  This function is used to shut down the communication                  */
  /*                                                                        */
  /*  possible return values:                                               */
  /*    NONE                                                                */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR   sdb;

  USIGN16                 retry_counter;


FUNCTION_BODY

  sdb.layer     = FMB;
  sdb.service   = FMB_EXIT;
  sdb.primitive = REQ;

  result = profi_snd_req_res (&sdb, &sdb, PB_FALSE);

  sprintf (s, "\nFMB_Exit.req [%d]", result);
  OUTPUT_STRING (s);

  /*------------------------------------------------------------------------*/

  retry_counter = 1000;

  do
  {
    PROFI_RCV_CON_IND ();

  } while ( (con_ind_sdb.service   != FMB_EXIT) &&
            (con_ind_sdb.primitive != CON     ) &&
            (con_ind_sdb.result    != POS     ) &&
            (retry_counter--                  ) );

  if (retry_counter)
  {
    sprintf (s, "FMB_Exit.con [+]");
    OUTPUT_STRING (s);
  }
  else
  {
    sprintf (s, "FMB_Exit.con [-]");
    OUTPUT_STRING (s);
  }

  /*------------------------------------------------------------------------*/

  result = profi_end ();

  sprintf (s, "\nTerminating controller program, result = %d", result);
  OUTPUT_STRING (s);

} /* exit_communication */

/****************************************************************************/

FUNCTION static VOID print_version
  (
  IN VOID
  )

  /*-------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                  */
  /*-------------------------------------------------------------------------*/
  /*                                                                         */
  /* prints program version on screen                                        */
  /*                                                                         */
  /* possible return values:                                                 */
  /*   - NONE                                                                */
  /*                                                                         */
  /*-------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf ("\n           PROFIBUS DP-Demo V 5.24 (C) 1995-2006 Softing AG\n");
  printf ("                               Softing AG\n");
  printf ("                          Richard-Reitzner-Allee 6\n");
  printf ("                             D-85540 Haar\n");
  printf ("               Tel: (++49) (0) 89 / 45 65 6 - 0 (-399 Fax)\n");
  printf ("                          http://www.softing.com\n");

}

/****************************************************************************/

FUNCTION static VOID print_papi_firmware_version
  (
  IN VOID
  )

  /*-------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                  */
  /*-------------------------------------------------------------------------*/
  /*                                                                         */
  /* prints firmware version on screen                                       */
  /*                                                                         */
  /* possible return values:                                                 */
  /*   - NONE                                                                */
  /*                                                                         */
  /*-------------------------------------------------------------------------*/
{
LOCAL_VARIABLES
  CSTRING FAR*     c;

  CSTRING   papi_version[VERSION_STRING_LENGTH];
  CSTRING   fw_version[VERSION_STRING_LENGTH];
  USIGN32   serial_device_number;

FUNCTION_BODY

  profi_get_versions((CSTRING FAR*) papi_version,(CSTRING FAR*) fw_version);
  profi_get_serial_device_number((USIGN32 FAR*)&serial_device_number);

  c=fw_version;
  do
    {
     c++;
     if (*c=='\n') *c=' '; /* replace <CR> by spaces */
    }
   while (*c!=0);


  printf("\n SN:            %lu",serial_device_number);
  printf("\n PAPI Version:  %s",papi_version);
  printf("\n FW Version:    %s\n\n",fw_version);

}

/****************************************************************************/

FUNCTION static VOID print_usage
  (
  IN    VOID
  )
/*--------------------------------------------------------------------*/
/* FUNCTIONAL_DESCRIPTION                                             */
/*--------------------------------------------------------------------*/
/*                                                                    */
/* prints program version and usage on screen                         */
/*                                                                    */
/* possible return values:                                            */
/*    - NONE                                                          */
/*                                                                    */
/*--------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

#if defined (WIN16) || defined (WIN32)
  printf ("Usage  : %s [options]\n\n", exe_fname);
#else
  printf ("Usage  : dpdemo [options]\n\n");
#endif

  printf ("options: \n");
#if defined (WIN32) || defined (_LINUX)
  printf ("-b<add>  = Board Number          (default: 0)\n");
  printf ("-h | -?  = display this help information\n\n");
#else
  printf ("-a<add>  = PC DPRAM address      (default: 0x%08lX)\n", H_DPR_BASE_ADDRESS);
  printf ("-i<add>  = PC IO address         (default: 0x%04X)\n" , IO_ADDRESS);
  printf ("-h | -?  = display this help information\n\n");
#endif

} /* print_usage */

/****************************************************************************/

FUNCTION static VOID check_options
  (
  IN char options []
  )
/*---------------------------------------------------------------------*/
/* FUNCTIONAL_DESCRIPTION                                              */
/*---------------------------------------------------------------------*/
/*                                                                     */
/* check command line parameters for valid options                     */
/*                                                                     */
/* valid options:                                                      */
/*    -a<DPR_ADDRESS>                                                  */
/*    -i<IO_ADDRESS>                                                   */
/*    -h | -?                      print usage                         */
/*                                                                     */
/* possible return values:                                             */
/*   - NONE                                                            */
/*                                                                     */
/*---------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  char    *token_ptr;

FUNCTION_BODY

  token_ptr = strtok (options, " \t\r");

  if (token_ptr != NULL)
  {
#if defined (WIN32) || defined (_LINUX)
    if (strncmp (token_ptr, "-b", 2) == 0)
    {
      if (sscanf (token_ptr, "-b%hi", &board_number) == 1)
      {
        if(!((board_number > 0 ) && (board_number <= 9 )))
        {
          board_number = BOARD_NUMBER;
        }
      }
      else
      {
        board_number = BOARD_NUMBER;
      }
    }
#else
    if (strncmp (token_ptr, "-a", 2) == 0)
    {
      if (sscanf (token_ptr, "-a0x%8lx", &dpr_base_address) != 1)
      {
        if (sscanf (token_ptr, "-a%10lu", &dpr_base_address) != 1)
        {
          dpr_base_address = (USIGN32) H_DPR_BASE_ADDRESS;
        }
      }
    }
    else if (strncmp (token_ptr, "-i", 2) == 0)
    {
      if (sscanf (token_ptr, "-i0x%4X", &io_address) != 1)
      {
        if (sscanf (token_ptr, "-i%5u", &io_address) != 1)
        {
          io_address = (USIGN16) IO_ADDRESS;
        }
      }
    }
#endif
    else if ( (strcmp (token_ptr, "-?") == 0) ||
              (strcmp (token_ptr, "-h") == 0) )
    {
      print_version ();
      print_usage ();

      exit (0);
    }
    else
    {
      print_version ();
      printf ("error: unknown command line parameter \"%s\" \n\n", token_ptr);
      print_usage ();

      exit (-1);
    }

    token_ptr = strtok (NULL, " \t\r");

  } /* if token_ptr */
}


/****************************************************************************/

FUNCTION static INT16 init_profi
  (
  IN    void
  )

/*---------------------------------------------------------------------*/
/* FUNCTIONAL_DESCRIPTION                                              */
/*---------------------------------------------------------------------*/
/*                                                                     */
/* This function is used to initialize the communication interface     */
/*                                                                     */
/* possible return values:                                             */
/*   - NONE                                                            */
/*                                                                     */
/*---------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  INT16  ret_val;

FUNCTION_BODY

#if defined (WIN32) || defined (_LINUX)
ret_val = profi_init((USIGN8) board_number,0,0);
#else
ret_val = init_profibus (dpr_base_address, io_address, PB_FALSE);
#endif

if (ret_val != E_OK) return(ret_val);

return(E_OK);
}

/****************************************************************************/
/****************************************************************************/

FUNCTION PUBLIC int main
        (
        IN  int  argc,
        IN  char **argv
        )
  /*------------------------------------------------------------------------*/
  /* FUNCTIONAL_DESCRIPTION                                                 */
  /*------------------------------------------------------------------------*/
  /*                                                                        */
  /* main routine of the program                                            */
  /*                                                                        */
  /*------------------------------------------------------------------------*/
{
LOCAL_VARIABLES
    int       i;

#if defined (WIN16) || defined (WIN32)
    char      dummy [_MAX_PATH];
#endif

FUNCTION_BODY

#if defined (WIN16) || defined (WIN32)
  _splitpath (argv [0], dummy, dummy, exe_fname, dummy);
#endif

  /* set_default_options */

#if defined (WIN32) || defined (_LINUX)
  board_number      = BOARD_NUMBER;
#else
  dpr_base_address  = (USIGN32) H_DPR_BASE_ADDRESS;
  io_address        = (USIGN16) IO_ADDRESS;
#endif

  load_firmware     = PB_FALSE;

  /* -- check command line ------------------------------------------------- */
  for (i = 1; i < argc; i++)
  {
    check_options (argv [i]);
  }

  print_version();

  printf("Settings:\n");
  printf("\nPress return to continue...\n\n");
  while (!kbhit());


  printf("This is a sample application running the PROFIBUS interface as DP-Master.\n");
  printf("It contains all necessary steps to set up a working DP communication.\n");
  printf("\n");
  printf("The DP slave used is a Softing DP Slave for PROFIboard/PROFI104 at address 3\n");
  printf("featuring 16 DOs.\n");
  printf("In case the user wants to access different slaves, the configuration\n");
  printf("data in DPDEMO.H has to be modified.\n");
  printf("\n");
  printf("The application performs the following actions:\n");
  printf("   - initialization of the PROFIBUS interface\n");
  printf("   - download of parameter sets to the PROFIBUS interface\n");
  printf("   - handling of DP communication\n");
  printf("\nIt may be terminated by pressing 'q' at any time\n");


  while (!kbhit());


  if (getch()=='q') return (0);

  printf ("\n\nInitializing controller...\n");

  result = init_profi ();

  if (result == E_OK)
  {
      printf("\nPROFIBUS interface initialization successful...\n\n");
      print_papi_firmware_version();
      printf("Press return to continue\n\n");
      while(!kbhit());
      atexit (exit_communication);

    handle_communication ();
    exit (0);

  } /* if init_profibus */
  else
  {
    sprintf (s, "\nError code = %d, aborting", result);
    OUTPUT_STRING (s);

  } /* else init_profibus */
  
   return (0);
} /* main */


