/******************************************************************************
*                                                                             *
*                                 SOFTING AG                                  *
*                          Richard-Reitzner-Allee 6                           *
*                                D-85540 Haar                                 *
*                        Phone: (++49)-(0)89-45656-0                          *
*                        Fax:   (++49)-(0)89-45656-399                        *
*                                                                             *
*                      Copyright (C) SOFTING AG 1995-2003                     *
*                              All Rights Reserved                            *
*                                                                             *
*******************************************************************************

FILE_NAME               DPV1DEMO.C

PROJECT_NAME            PROFIBUS

MODULE                  DPV1DEMO

COMPONENT_LIBRARY       --------

AUTHOR                  SOFTING AG

VERSION                 5.24

DATE                    31.07.2001

STATUS                  finished

FUNCTIONAL_MODULE_DESCRIPTION

DP/V1 Demo Application

RELATED_DOCUMENTS

============================================================================*/
#include <keywords.h>

INCLUDES

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#if defined (WIN16) || defined (WIN32)
#include <windows.h>
#include <conio.h>
#include <dos.h>
#else
#ifndef _LINUX
#include <graph.h>
#endif
#endif

#ifdef _LINUX
#include <unistd.h>
#include <ctype.h>
#endif

#include "pb_type.h"
#include "pb_conf.h"
#include "pb_if.h"
#include "pb_err.h"
#include "pb_fmb.h"
#include "pb_dp.h"
#include "dpv1demo.h"
#include "assert.h"

GLOBAL_DEFINES

LOCAL_DEFINES

/* --- default controller addresses --------------------------------------- */
#if defined (WIN32) || defined (_LINUX)
#define BOARD_NUMBER                    0
#else
#define H_DPR_BASE_ADDRESS              0xD0000000
#define IO_ADDRESS                      0x0240
#endif

#define DEFAULT_MASTER                  1
#define DEFAULT_SLAVE                   5

#define STRING_DP_ABORT_INSTANCE_FDL    "FDL"
#define STRING_DP_ABORT_INSTANCE_DDLM   "DDLM"
#define STRING_DP_ABORT_INSTANCE_USER   "USER"

EXPORT_TYPEDEFS

LOCAL_TYPEDEFS

typedef enum
{
  STATE_STARTUP,
  STATE_BOARD_OK,
  STATE_USER_START,
  STATE_FMB_WAIT_CON,
  STATE_FMB_CONFIGURATION_OK,
  STATE_FMB_WAIT_PAR,
  STATE_FMB_BUSPARAMETER_OK,
  STATE_DP_USER_START,
  STATE_INIT_MASTER_WAIT_CON,
  STATE_DP_INIT_MASTER_OK,
  STATE_WAIT_FOR_CON,
  STATE_IDLE

} PROGRAM_STATE;

FUNCTION_DECLARATIONS

EXPORT_DATA

IMPORT_DATA

LOCAL_DATA

static PROGRAM_STATE           global_state;
static int                     bWaitForInput = 0;

#if defined (WIN16) || defined (WIN32)
static char                    exe_fname [_MAX_FNAME];  /* name of .exe file */
#endif

#if defined (WIN32) || defined (_LINUX)
static short unsigned int      board_number;
#else
static long  unsigned int      dpr_base_address;
static short unsigned int      io_address;
#endif


static USIGN8                  ts;
static USIGN16                 gateway_flag;
static USIGN16                 remote_address;
static USIGN16                 max_length;

/****************************************************************************/
/**** AUXILIARY USER INTERFACE FUNCTIONS ************************************/
/****************************************************************************/

FUNCTION static VOID print_version(
           VOID
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print Version String                                                     */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY
#ifdef DOS16
  _clearscreen( _GCLEARSCREEN );
#endif
  printf ("\n\n");
  printf ("            PROFIBUS DP/V1 Demo V 5.24 (C) 1997-2003 Softing AG\n\n");
  printf ("                                Softing AG\n");
  printf ("                          Richard-Reitzner-Allee 6\n");
  printf ("                             D-85540 Haar\n\n");
  printf ("                     Phone: (++49) (0) 89 / 45 65 6 - 0\n");
  printf ("                     Fax  : (++49) (0) 89 / 45 65 6 - 399\n\n");
  fflush(stdout);

} /* print_version */

/*********************** PRINT_MENU *****************************************/

FUNCTION static VOID print_menu(
           VOID
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print Version String                                                     */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY
    
  if(bWaitForInput)
    return;

  switch (global_state)
  {
    case STATE_STARTUP:
      printf ("\n\n");
      printf (" <X> ....... Exit      \n");
      printf (" <G> ....... Start FMB with parameters for Pepperl+Fuchs Coupler\n");
      printf (" <R> ....... Start FMB with parameters for RS485\n");
      printf ("\n");
      bWaitForInput = 1;
      break;

    case STATE_FMB_BUSPARAMETER_OK:
      printf ("\n\n");
      printf (" <X> ....... Exit     \n");
      printf (" <D> ....... Start DP \n");
      printf ("\n");
      bWaitForInput = 1;
      break;

    case STATE_IDLE:
      printf ("\n\n");
      printf (" <X> ....... Exit     \n");
      printf (" <I> ....... Initialize Connection \n");
      printf (" <R> ....... Read Object in Slave \n");
      printf (" <W> ....... Write Object in Slave \n");
      printf (" <A> ....... Abort Connection \n");
      printf ("\n");
      bWaitForInput = 1;
      break;
      
    default: break;
  }
  fflush(stdout);
} /* print_menu */

/*********************** ERROR_EXIT *****************************************/

FUNCTION static VOID error_exit(
           IN char FAR*  msg
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Exit Program with Message                                                */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf ("\n%s\n",msg);
  fflush(stdout);
  profi_end();
  exit(1);

} /* error_exit */

/*********************** PRINT_IF_ERROR *************************************/

FUNCTION static VOID print_if_error(
           IN USIGN8 primitive,
           IN INT16  error
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print Interface error, get additional data if available                  */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_EXCEPTION exception;
  USIGN16     size = sizeof(T_EXCEPTION);

FUNCTION_BODY

  if (primitive == REQ || primitive == RES)
  {
    printf("\nprofi_snd_req_res failed\n");
  }
  else if (primitive == IND || primitive == CON)
  {
    printf("\nprofi_rcv_con_ind returns with error\n");
  }

  switch (error)
  {
    case E_IF_FATAL_ERROR:
         printf("  unrecoverable error on board     \n");
         while (profi_get_data(ID_EXCEPTION_IMAGE,
                               0,
                               (USIGN16 FAR*) &size,
                               (VOID FAR*) &exception
                              ) == E_IF_SERVICE_CONSTR_CONFLICT);

         printf("  task_id : %d (dez) %X (hex)\n",exception.task_id,exception.task_id);
         printf("  par1    : %d (dez) %X (hex)\n",exception.par1,exception.par1);
         printf("  par2    : %d (dez) %X (hex)\n",exception.par2,exception.par2);
         printf("  par3    : %d (dez) %X (hex)\n",exception.par3,exception.par3);
         break;

    case E_IF_INIT_INVALID_PARAMETER:    printf("  E_IF_INIT_INVALID_PARAMETER");                          break;
    case E_IF_LOADER_ERROR:              printf("  E_IF_LOADER_ERROR");                                    break;
    case E_IF_NO_CNTRL_RES:              printf("  E_IF_NO_CNTRL_RES");                                    break;
    case E_IF_INVALID_CNTRL_TYPE_VERSION:printf("  E_IF_INVALID_CNTRL_TYPE_VERSION");                      break;
    case E_IF_INVALID_LAYER:             printf("  E_IF_INVALID_LAYER");                                   break;
    case E_IF_INVALID_SERVICE:           printf("  E_IF_INVALID_SERVICE");                                 break;
    case E_IF_INVALID_PRIMITIVE:         printf("  E_IF_INVALID_PRIMITIVE");                               break;
    case E_IF_INVALID_DATA_SIZE:         printf("  E_IF_INVALID_DATA_SIZE");                               break;
    case E_IF_INVALID_COMM_REF:          printf("  E_IF_INVALID_COMM_REF");                                break;
    case E_IF_INVALID_CMI_CALL:          printf("  E_IF_INVALID_CMI_CALL");                                break;
    case E_IF_CMI_ERROR:                 printf("  E_IF_CMI_ERROR");                                       break;
    case E_IF_RESOURCE_UNAVAILABLE:      printf("  E_IF_RESOURCE_UNAVAILABLE");                            break;
    case E_IF_NO_PARALLEL_SERVICES:      printf("  E_IF_NO_PARALLEL_SERVICES");                            break;
    case E_IF_SERVICE_CONSTR_CONFLICT:   printf("  E_IF_SERVICE_CONSTR_CONFLICT");                         break;
    case E_IF_SERVICE_NOT_SUPPORTED:     printf("  E_IF_SERVICE_NOT_SUPPORTED");                           break;
    case E_IF_SERVICE_NOT_EXECUTABLE:    printf("  E_IF_SERVICE_NOT_EXECUTABLE");                          break;
    case E_IF_INVALID_VERSION:           printf("  E_IF_INVALID_VERSION");                                 break;
    case E_IF_NO_CNTRL_PRESENT:          printf("  E_IF_NO_CNTRL_PRESENT (0x%02x)",profi_get_last_error());break;
    case E_IF_INVALID_PARAMETER:         printf("  E_IF_INVALID_PARAMETER");                               break;
    case E_IF_INIT_FAILED:               printf("  E_IF_INIT_FAILED (0x%02x)",profi_get_last_error());     break;
    case E_IF_EXIT_FAILED:               printf("  E_IF_EXIT_FAILED (0x%02x)",profi_get_last_error());     break;
    case E_IF_OS_ERROR:                  printf("  E_IF_OS_ERROR    (0x%02x)",profi_get_last_error());     break;
    default:                             printf("  UNKNOWN ERROR: %d",error);                              break;
  }
  fflush(stdout);
} /* print_if_error */

/*********************** PRINT_IF_ERROR *************************************/

FUNCTION static VOID print_usage(
  IN    VOID
  )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Prints out a short help for the command-line options                     */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

#if defined (WIN16) || defined (WIN32)
  printf ("Usage  : %s [options] \n\n", exe_fname);
#else
  printf ("Usage  : dpv1demo [options]\n\n");
#endif

  printf ("options: \n");
#if defined (WIN32) || defined (_LINUX)
  printf ("-b<add>  = Board Number          (default: 0\n");
  printf ("-s<add>  = Slave Address         (default: %d\n", DEFAULT_SLAVE);
  printf ("-h | -?  = display this help information\n\n");
#else
  printf ("-a<add>  = PC DPRAM address      (default: 0x%08lX)\n", H_DPR_BASE_ADDRESS);
  printf ("-i<add>  = PC IO address         (default: 0x%04X)\n" , IO_ADDRESS);
  printf ("-s<add>  = Slave Address         (default: %d\n"      , DEFAULT_SLAVE);
  printf ("-h | -?  = display this help information\n\n");
#endif

  fflush(stdout);

} /* print_usage */

/******************* PRINT_PAPI_FIRMWARE_VERSION ****************************/

FUNCTION static VOID print_papi_firmware_version(
           VOID
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print Papi-Version and Firmware-Version                                  */
/* ------------------------------------------------------------------------ */
{
  LOCAL_VARIABLES

  CSTRING   papi_version[VERSION_STRING_LENGTH];
  CSTRING   fw_version[VERSION_STRING_LENGTH];
  USIGN32   serial_device_number;

  FUNCTION_BODY

  profi_get_versions((CSTRING FAR*) papi_version,(CSTRING FAR*) fw_version);
  profi_get_serial_device_number((USIGN32 FAR*)&serial_device_number);

  printf("\n Serial Device Number: %lu",serial_device_number);
  printf("\n PAPI Version:  %s",papi_version);
  printf("\n FW Version:    %s",fw_version);
  printf("\n\n");
  fflush(stdout);

} /* print_papi_firmware_version */

/****************************************************************************/
/**** REQUEST FUNCTIONS: COMMON FUNCTIONS ***********************************/
/****************************************************************************/

FUNCTION static INT16 fmb_set_configuration(
           VOID
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Start Profibus DP                                                        */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_FMB_SET_CONFIGURATION_REQ  fmb_config_req;
  T_PROFI_SERVICE_DESCR        sdb;
  T_FMB_CONFIG_DP FAR*         config_dp_ptr;

FUNCTION_BODY

  /* Do only activate DP, but no other module */

  fmb_config_req.fms_active                = PB_FALSE;
  fmb_config_req.dp_active                 = PB_TRUE;
  fmb_config_req.fdlif_active              = PB_FALSE;
  fmb_config_req.sm7_active                = PB_FALSE;
  fmb_config_req.fdl_evt_receiver          = FMB_USR;
  fmb_config_req.data_buffer_length        = 255;

  /* Set DP-Parameter-Block */

  config_dp_ptr      = (T_FMB_CONFIG_DP FAR*) &(fmb_config_req.dp);

  config_dp_ptr->max_number_slaves       = 125;
  config_dp_ptr->max_slave_output_len    = 32;
  config_dp_ptr->max_slave_input_len     = 32;
  config_dp_ptr->max_slave_diag_entries  = 300;
  config_dp_ptr->max_slave_diag_len      = 32;
  config_dp_ptr->max_bus_para_len        = 128;
  config_dp_ptr->max_slave_para_len      = 128;

  /* Set Description block */

  sdb.comm_ref   = 0;
  sdb.layer      = FMB;
  sdb.service    = FMB_SET_CONFIGURATION;
  sdb.primitive  = REQ;

  /* Send Request */

  return(profi_snd_req_res(&sdb,&fmb_config_req,PB_TRUE));

} /* fmb_set_configuration */

/*********************** FMB_SET_BUSPARAMETER ******************************/

FUNCTION static INT16 fmb_set_busparameter(
           IN USIGN8 ts
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Set Busparameters for a DP/V1 network                                    */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_FMB_SET_BUSPARAMETER_REQ  bus_par_req;
  T_PROFI_SERVICE_DESCR       sdb;

FUNCTION_BODY

  /* set data block parameters */

  bus_par_req.loc_add         = ts;

  bus_par_req.in_ring_desired = 255;
  bus_par_req.baud_rate       = 2;
  bus_par_req.loc_segm        = 255;
  bus_par_req.medium_red      = 0;

  if (gateway_flag ==0)
  {
    /* Baudrate 93,75 with no Coupler to IEC-31,25 */
    bus_par_req.tset            = 1;
    bus_par_req.tsl             = 100;
    bus_par_req.min_tsdr        = 11;
    bus_par_req.max_tsdr        = 60;
    bus_par_req.tqui            = 0;
    bus_par_req.max_retry_limit = 1;
  }
  else
  {
    /* Baudrate 93,75 with Pepperl+Fuchs Coupler RS485 to IEC-31,25 */
    bus_par_req.tset            = 200;
    bus_par_req.tsl             = 8000;
    bus_par_req.min_tsdr        = 250;
    bus_par_req.max_tsdr        = 1000;
    bus_par_req.tqui            = 150;
    bus_par_req.max_retry_limit = 4;
  }

  bus_par_req.ttr             = 100000;
  bus_par_req.g               = 1;
  bus_par_req.hsa             = 126;

  /* Set Description block */

  sdb.comm_ref   = 0;
  sdb.layer      = FMB;
  sdb.service    = FMB_SET_BUSPARAMETER;
  sdb.primitive  = REQ;

  /* Send Request */

  return(profi_snd_req_res(&sdb,&bus_par_req,PB_TRUE));

} /* fmb_set_busparameter */

/****************************************************************************/
/**** REQUEST FUNCTIONS: DPV1 ***********************************************/
/****************************************************************************/

FUNCTION static INT16 dp_initiate_req(
           IN USIGN16 send_timeout
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Start a DP/V1 connection                                                 */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  char initiate_data_buffer[255];

  T_PROFI_SERVICE_DESCR   sdb;
  T_DP_INITIATE_REQ FAR*  initiate_req_ptr = (T_DP_INITIATE_REQ FAR*) &initiate_data_buffer[0];


FUNCTION_BODY

  /* Zero all standard parameters, especially the reserved ones */

  memset((VOID FAR*)initiate_req_ptr,0,sizeof(T_DP_INITIATE_REQ));

  /* Set mandatory parameters */

  initiate_req_ptr->rem_add               = (USIGN8) remote_address;
  initiate_req_ptr->send_timeout          = send_timeout;
  initiate_req_ptr->features_supported[0] = 1;        /* use dp_read and dp_write */
  initiate_req_ptr->profile_ident_number  = 0;            /* no profile supported */

  /* Set additional address parameters */
  /* Destination address: standard address as we do not know the slave */
  /* Source address: use something more complicated to demonstrate the macros from pb_def.h */

  initiate_req_ptr->add_addr_param.d_type = 0;
  initiate_req_ptr->add_addr_param.d_len  = 2;

  initiate_req_ptr->add_addr_param.s_type = 0;
  initiate_req_ptr->add_addr_param.s_len  = 2;

  /* Before using the macros, s_len MUST be set */

  DP_INITIATE_S_ADDR(initiate_req_ptr)->api = 0;
  DP_INITIATE_D_ADDR(initiate_req_ptr)->api = 0;

  DP_INITIATE_S_ADDR(initiate_req_ptr)->scl = 0;
  DP_INITIATE_D_ADDR(initiate_req_ptr)->scl = 0;

  /* Set Description block */

  sdb.comm_ref   = 0;
  sdb.layer      = DP;
  sdb.service    = DP_INITIATE;
  sdb.primitive  = REQ;
  sdb.invoke_id  = 0;

  /* Send Request */

  return(profi_snd_req_res(&sdb,initiate_req_ptr,PB_TRUE));

} /* dp_initiate_req */

/***************************** DP_READ_REQ **********************************/

FUNCTION static INT16 dp_read_req(
           IN USIGN8 slot_number,
           IN USIGN8 index
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Read an Object from the slave with DP/V1 services                        */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR sdb;
  T_DP_READ_REQ read_req;

FUNCTION_BODY

  /* Zero all standard parameters */

  memset((VOID FAR*)&read_req,0,sizeof(T_DP_READ_REQ));

  /* Set mandatory parameters */

  read_req.slot_number = slot_number;
  read_req.index       = index;
  read_req.length      = (USIGN8) max_length; /* do not exceed the connections maximum */

  /* Set Description block */

  sdb.comm_ref   = 0;
  sdb.layer      = DP;
  sdb.service    = DP_READ;
  sdb.primitive  = REQ;
  sdb.invoke_id  = 0;

  /* Send Request */

  return(profi_snd_req_res(&sdb,&read_req,PB_TRUE));

} /* dp_read_req */

/***************************** DP_WRITE_REQ *********************************/

FUNCTION static INT16 dp_write_req(
           IN USIGN8 slot_number,
           IN USIGN8 index,
           IN USIGN8 len,
           IN USIGN8 *data
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Write ABCD into a slave object with DP/V1 services                       */
/* ------------------------------------------------------------------------ */
{
  LOCAL_VARIABLES

      int i;
   char my_write_data[sizeof(T_DP_WRITE_REQ)+256+4];

  T_PROFI_SERVICE_DESCR sdb;
  T_DP_WRITE_REQ FAR* write_req_ptr = (T_DP_WRITE_REQ FAR*) &my_write_data[0];

  FUNCTION_BODY

      /* Zero all standard parameters */

      /* Set mandatory parameters */

  write_req_ptr->slot_number = slot_number;
  write_req_ptr->index       = index;
  write_req_ptr->length      = len;
  
  memcpy(&my_write_data[4],data,len);

  /* Set the Data to write into the slave object     */
  /* write-data start behind the standard-parameters */

  //memcpy ((VOID FAR*)(write_req_ptr+1),data,len);

  /* Set Description block */

  sdb.comm_ref   = 0;
  sdb.layer      = DP;
  sdb.service    = DP_WRITE;
  sdb.primitive  = REQ;
  sdb.invoke_id  = 0;
  
  printf("writing %u bytes to slot %u, index %u\n", len, slot_number, index);
  
  printf("data: ");
  for(i=0; i<len; i++)
  {
    printf("0x%02x ", my_write_data[4+i]);
  }
  
  printf("\n");
  fflush(stdout);

  /* Send Request */

  return(profi_snd_req_res(&sdb,write_req_ptr,PB_TRUE));

} /* dp_write_req */

/***************************** DP_ABORT_REQ *********************************/

FUNCTION static INT16 dp_abort_req(
           VOID
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Abort a DP/V1 connection                                                 */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR sdb;
  T_DP_ABORT_REQ abort_req;

FUNCTION_BODY

  /* Zero all standard parameters */

  memset((VOID FAR*)&abort_req,0,sizeof(T_DP_ABORT_REQ));

  /* Set mandatory parameters */

  abort_req.subnet     = DP_ABORT_SUBNET_NO;
  abort_req.reason     = 0;

  /* Set Description block */

  sdb.comm_ref   = 0;
  sdb.layer      = DP;
  sdb.service    = DP_ABORT;
  sdb.primitive  = REQ;
  sdb.invoke_id  = 0;

  /* Send Request */

  return(profi_snd_req_res(&sdb,&abort_req,PB_TRUE));

} /* dp_abort_req */

/****************************************************************************/
/**** REQUEST FUNCTIONS: STANDARD DP ****************************************/
/****************************************************************************/

FUNCTION static INT16 dp_act_param_loc_req(
            VOID
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Set the DP-Master into state STOP                                        */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR      sdb;
  char                       act_param_buffer[255];
  T_DP_ACT_PARAM_REQ FAR*    act_param_loc_req_ptr = (T_DP_ACT_PARAM_REQ FAR*) &act_param_buffer[0];

FUNCTION_BODY

  act_param_loc_req_ptr->area_code = DP_AREA_SET_MODE;
  act_param_loc_req_ptr->activate  = DP_OP_MODE_STOP;

  /* Set Description block */

  sdb.layer       = DP;
  sdb.service     = DP_ACT_PARAM_LOC;
  sdb.primitive   = REQ;

  /* Send Request */

  return(profi_snd_req_res (&sdb, act_param_loc_req_ptr, PB_FALSE));

} /* dp_act_param_loc_req */

/************************* DP_INIT_MASTER_REQ *******************************/

FUNCTION static INT16 dp_init_master_req(
           VOID
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Start the DP-Task on the Controller                                      */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR      sdb;
  T_DP_INIT_MASTER_REQ       init_master_req;

FUNCTION_BODY

  init_master_req.master_default_address = ts;
  init_master_req.master_class2          = PB_TRUE;
  init_master_req.lowest_slave_address   = 0;
  init_master_req.slave_io_address_mode  = DP_AAM_ARRAY;
  init_master_req.clear_outputs          = PB_FALSE;
  init_master_req.auto_remote_services   = DP_AUTO_REMOTE_SERVICES;
  init_master_req.cyclic_data_transfer   = PB_TRUE;

  memset((VOID FAR*)&init_master_req.master_class2_name[0],' ',32);

  /* Set Description block */

  sdb.layer     = DP;
  sdb.service   = DP_INIT_MASTER;
  sdb.primitive = REQ;

  /* Send Request */

  return(profi_snd_req_res (&sdb, &init_master_req, PB_FALSE));

} /* dp_init_master_req */

/****************************************************************************/
/**** CONVERTING DEFINES INTO STRINGS ***************************************/
/****************************************************************************/

FUNCTION static char FAR* error_name(
           IN USIGN16 code
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Convert a DP-Error-Code into a corresponding String                      */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  switch (code)
  {
    case E_DP_OK: return("E_DP_OK");
    case E_DP_UE: return("E_DP_UE");
    case E_DP_RR: return("E_DP_RR");
    case E_DP_RS: return("E_DP_RS");
    case E_DP_RA: return("E_DP_RA");
    case E_DP_NA: return("E_DP_NA");
    case E_DP_DS: return("E_DP_DS");
    case E_DP_NO: return("E_DP_NO");
    case E_DP_LR: return("E_DP_LR");
    case E_DP_IV: return("E_DP_IV");
    case E_DP_TO: return("E_DP_TO");
    case E_DP_FE: return("E_DP_FE");
    case E_DP_NI: return("E_DP_NI");
    case E_DP_AD: return("E_DP_AD");
    case E_DP_EA: return("E_DP_EA");
    case E_DP_LE: return("E_DP_LE");
    case E_DP_RE: return("E_DP_RE");
    case E_DP_IP: return("E_DP_IP");
    case E_DP_SC: return("E_DP_SC");
    case E_DP_SE: return("E_DP_SE");
    case E_DP_NE: return("E_DP_NE");
    case E_DP_DI: return("E_DP_DI");
    case E_DP_NC: return("E_DP_NC");
    default     : return("UNKNOWN ERROR");
  }
} /* fmb_name */

/****************************** FMB_NAME ************************************/

FUNCTION static char FAR* fmb_name(
           IN USIGN8 service
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Convert a FMB-Servic-Define into a corresponding String                  */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  switch (service)
  {
    case FMB_SET_VALUE         : return ("FMB_SET_VALUE");
    case FMB_READ_VALUE        : return ("FMB_READ_VALUE");
    case FMB_LSAP_STATUS       : return ("FMB_LSAP_STATUS");
    case FMB_FM2_EVENT         : return ("FMB_FM2_EVENT");
    case FMB_RESET             : return ("FMB_RESET");
    case FMB_EXIT              : return ("FMB_EXIT");
    case FMB_SET_BUSPARAMETER  : return ("FMB_SET_BUSPARAMETER");
    case FMB_READ_BUSPARAMETER : return ("FMB_READ_BUSPARAMETER");
    case FMB_GET_LIVE_LIST     : return ("FMB_GET_LIVE_LIST");
    case FMB_SET_CONFIGURATION : return ("FMB_SET_CONFIGURATION");
    case FMB_EXCEPTION         : return ("FMB_EXCEPTION");
    default                    : return ("UNKNOWN SERVICE");
  }
} /* fmb_name */

/****************************** DP_NAME *************************************/

FUNCTION static char FAR* dp_name(
           IN USIGN8 service
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Convert a DP(/V1)-Servic-Define into a corresponding String              */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  switch (service)
  {
    /* standard DP services */
    case DP_INIT_MASTER        : return ("DP_INIT_MASTER");
    case DP_ACT_PARAM_LOC      : return ("DP_ACT_PARAM_LOC");

    /* DP/V1 services */
    case DP_INITIATE           : return ("DP_INITIATE");
    case DP_READ               : return ("DP_READ");
    case DP_WRITE              : return ("DP_WRITE");
    case DP_ABORT              : return ("DP_ABORT");

    /* anything else */
    default                    : return ("UNKNOWN SERVICE");
  }
} /* dp_name */

/****************************************************************************/
/**** SCREEN OUTPUT OF RECEIVED DATA ****************************************/
/****************************************************************************/

FUNCTION static VOID print_dp_initiate_data(
           IN T_DP_INITIATE_CON FAR*   initiate_con_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print the received data from an initiate-response                        */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  INT16 i;

FUNCTION_BODY

  print_pos_con(dp_name(DP_INITIATE));

  max_length = initiate_con_ptr->max_len_data_unit;

  printf("Max-Len-Data-Unit: %d\n", max_length);
  printf("Source Api/SCL   : 0x%02x/0x%02x\n",
          DP_INITIATE_S_ADDR(initiate_con_ptr)->api,
          DP_INITIATE_S_ADDR(initiate_con_ptr)->scl);
  printf("Dest. Api/SCL    : 0x%02x/0x%02x\n",
          DP_INITIATE_D_ADDR(initiate_con_ptr)->api,
          DP_INITIATE_D_ADDR(initiate_con_ptr)->scl);

  /* Print Source Network-Address (=SLAVE), if there is one */

  if (initiate_con_ptr->add_addr_param.s_type)
  {
    printf("Source Address   : ");
    for (i=0;i<initiate_con_ptr->add_addr_param.s_len-2;i++)
    {
      if (i==6) printf(" ");
      printf("%c",(int)DP_INITIATE_S_ADDR(initiate_con_ptr)->network_address[i]);
    }
    printf("\n");
  }

  /* Print Destination Network-Address (=MASTER), if there is one */

  if (initiate_con_ptr->add_addr_param.d_type)
  {
    printf("Dest. Address    : ");
    for (i=0;i<initiate_con_ptr->add_addr_param.d_len-2;i++)
    {
      if (i==6) printf(" ");
      printf("%c",(int)DP_INITIATE_D_ADDR(initiate_con_ptr)->network_address[i]);
    }
    printf("\n");
  }
  fflush(stdout);
} /* print_dp_initiate_data */

/*************************** PRINT_DP_READ_DATA *****************************/

FUNCTION static VOID print_dp_read_data(
           IN T_DP_READ_CON FAR*   read_con_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print DP/V1-Read-Confirmation and the first 8 Byte of the received data  */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  USIGN16      i;
  USIGN8       max_print;
  USIGN8 FAR*  data;

FUNCTION_BODY

  print_pos_con(dp_name(DP_READ));
  printf("Slot_number          : %d\n",read_con_ptr->slot_number);
  printf("Index                : %d\n",read_con_ptr->index);
  printf("Bytes received       : %d\n",read_con_ptr->length);
  printf("Data                 : ");

  max_print = read_con_ptr->length;
  data = (USIGN8 FAR*)(read_con_ptr+1); /* Read Data start behind the mandatory data */

  if (read_con_ptr->length > 8) max_print = 8;                /* Dont print too much */

  for (i=0;i<max_print;i++)
  {
    printf("%2x ",(USIGN16) *data);
    data++;
  }

  if (read_con_ptr->length > 8) printf ("....");     /* Just in case we cut some byte */
  printf("\n");
  
  fflush(stdout);

} /* print_dp_read_data */

/*************************** PRINT_DP_WRITE_DATA ****************************/

FUNCTION static VOID print_dp_write_data(
           IN T_DP_WRITE_CON FAR*   write_con_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print DP/V1-Write-Confirmation                                           */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  print_pos_con(dp_name(DP_WRITE));
  printf("Slot_number          : %d\n",write_con_ptr->slot_number);
  printf("Index                : %d\n",write_con_ptr->index);
  printf("Bytes written        : %d\n",write_con_ptr->length);
  fflush(stdout);

} /* print_dp_write_data */

/*************************** PRINT_DP_ABORT_DATA ****************************/

FUNCTION static VOID print_dp_abort_data(
           IN T_DP_ABORT_IND FAR*   abort_ind_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print DP/V1-Abort-Data and give hints for the errors which occur         */
/* rather often                                                             */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  USIGN16 reason = abort_ind_ptr->reason;

FUNCTION_BODY

  printf("\n%s.ind\n",dp_name(DP_ABORT));
  printf("Local    : 0x%02x\n", (USIGN16)abort_ind_ptr->locally_generated);
  printf("Instance : 0x%02x", (USIGN16)((abort_ind_ptr->reason & DP_ABORT_INSTANCE_MASK)>>6));
  switch (abort_ind_ptr->reason & DP_ABORT_INSTANCE_MASK)
  {
    case DP_ABORT_INSTANCE_FDL  :   printf("(" STRING_DP_ABORT_INSTANCE_FDL  ")"); break;
    case DP_ABORT_INSTANCE_DDLM :   printf("(" STRING_DP_ABORT_INSTANCE_DDLM ")"); break;
    case DP_ABORT_INSTANCE_USER :   printf("(" STRING_DP_ABORT_INSTANCE_USER ")"); break;
  }
  printf("\n");
  printf("Reason   : 0x%02x\n", reason & DP_ABORT_REASON_MASK);
  printf("Subnet   : 0x%02x\n", (USIGN16) abort_ind_ptr->subnet);
  printf("Detail   : 0x%02x\n", (USIGN16) abort_ind_ptr->additional_detail);

  switch (reason)
  {
    /* catch the very most common errors! */
    case DP_ABORT_FDL_RS:
      printf("\nEither this is not a DP/V1-Slave or\n");
      printf("the slave has no more resources for (another) connection!\n");
      break;

    case DP_ABORT_FDL_NA:
      printf("\nThere seems to be no slave with this address!\n");
      printf("This problem can also occur, when the cabling is not oK.");
      break;

    case DP_ABORT_DDLM_ABT_STO:
      printf("\nThe send_timeout in the initiate_request was too small.\n");
      printf("Please repeat the initiate with send_timeout>=%d!\n",abort_ind_ptr->additional_detail);
      break;
  }
  fflush(stdout);
} /* print_dp_abort_data */

/************************* PRINT_FMB_IND_DATA ********************************/

FUNCTION static VOID print_fmb_ind_data(
           IN T_FMB_FM2_EVENT_IND FAR*  event
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print FMB-Event-Data                                                     */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf("\nFMB_EVENT.ind:\n");
  printf("  Reason       : 0x%02x    \n", event->reason);
  bWaitForInput = 0;
  fflush(stdout);
  

} /* print_fmb_ind_data */

/************************ PRINT_POS_CON *************************************/

FUNCTION static VOID print_pos_con(
           IN char FAR* service_name
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print a positive confirmation                                            */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf("\n%s.con(+)\n",service_name);
  fflush(stdout);

} /* print_pos_con */

/************************ PRINT_NEG_DPV1_CON ********************************/

FUNCTION static VOID print_neg_dpv1_con(
           IN char FAR*           service_name,
           IN T_DP_ERROR_CON FAR* error_con_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print a negative DP/V1 confirmation and its parameters                   */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf("\n%s.con(-)                 \n",service_name);
  printf("  Status       : 0x%04x (%s)\n",error_con_ptr->status,error_name(error_con_ptr->status));
  printf("  Error-Decode : 0x%02x     \n",error_con_ptr->error_decode);
  printf("  Error-Code-1 : 0x%02x     \n",error_con_ptr->error_code_1);
  printf("  Error-Code-2 : 0x%02x     \n",error_con_ptr->error_code_2);
  fflush(stdout);

} /* print_neg_dpv1_con */

/************************ PRINT_NEG_FMB_CON ********************************/

FUNCTION static VOID print_neg_fmb_con(
           IN char FAR*       service_name,
           IN T_ERROR FAR*    error_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Print a negative FMB confirmation and its parameters                     */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf("%s.con(-)           \n",service_name);
  printf("  Class-Code  : 0x%02x\n",error_ptr->class_code);
  printf("  Add-Detail  : 0x%02x\n",error_ptr->add_detail);
  fflush(stdout);

} /* print_neg_fmb_con */

/****************************************************************************/
/**** RECEIVE FUNCTIONS *****************************************************/
/****************************************************************************/

FUNCTION static VOID dp_ind_con(
           IN T_PROFI_SERVICE_DESCR FAR* sdb_ptr,
           IN char FAR*                  data_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Test a received DP-confirmation / indication and set the                 */
/* program state accordingly                                                */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  char FAR* service_name;

FUNCTION_BODY

  service_name = dp_name(sdb_ptr->service);
  if (sdb_ptr->result == POS)
  {
    switch (sdb_ptr->service)
    {

      case DP_INIT_MASTER:
        assert(sdb_ptr->primitive == CON);
        print_pos_con(service_name);
        global_state++;
        break;

      case DP_ACT_PARAM_LOC:
        assert(sdb_ptr->primitive == CON);
        print_pos_con(service_name);
        global_state++;
        break;

      case DP_INITIATE:
        assert(sdb_ptr->primitive == CON);
        print_dp_initiate_data ((T_DP_INITIATE_CON FAR*)data_ptr);
        global_state = STATE_IDLE;
        break;

      case DP_READ:
        assert(sdb_ptr->primitive == CON);
        print_dp_read_data ((T_DP_READ_CON FAR*)data_ptr);
        global_state = STATE_IDLE;
        break;

      case DP_WRITE:
        assert(sdb_ptr->primitive == CON);
        print_dp_write_data ((T_DP_WRITE_CON FAR*)data_ptr);
        global_state = STATE_IDLE;
        break;

      case DP_ABORT:
        assert(sdb_ptr->primitive == IND);
        print_dp_abort_data ((T_DP_ABORT_IND FAR*)data_ptr);
        global_state = STATE_IDLE;
        break;

      default:
        error_exit ("Unknown Service");
        break;
    }
  }
  else
  {
    switch (sdb_ptr->service)
    {
      case DP_INITIATE:
      case DP_READ:
      case DP_WRITE:
        assert(sdb_ptr->primitive == CON);
        print_neg_dpv1_con (service_name,(T_DP_ERROR_CON FAR*)data_ptr);
        global_state = STATE_IDLE;
        break;

      case DP_INIT_MASTER:
      case DP_ACT_PARAM_LOC:
        error_exit(service_name);
        break;

      default:
        error_exit ("Unknown Service");
        break;
    }
  }
}

/************************ FMB_IND_CON ************************************/

FUNCTION static VOID fmb_ind_con(
           IN T_PROFI_SERVICE_DESCR FAR* sdb_ptr,
           IN char FAR*                  data_ptr
         )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Test a received FMB-indication and increment the program state           */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  char FAR* service_name;

FUNCTION_BODY

  service_name = fmb_name (sdb_ptr->service);

  switch (sdb_ptr->primitive)
  {
    case CON:
      global_state++;
      if (sdb_ptr->result == POS)
      {
        print_pos_con (service_name);
        if (sdb_ptr->service == FMB_EXIT)
        {
          error_exit("Program terminated after FMB_EXIT");
        }
      }
      else
      {
        print_neg_fmb_con (service_name,(T_ERROR FAR*)data_ptr);
        error_exit("Fatal FMB-Error");
      }
      break;

    case IND:
      print_fmb_ind_data ((T_FMB_FM2_EVENT_IND FAR*) data_ptr);
      break;
  }
}

/************************ WAIT_FOR_CNF_IND ***********************************/

FUNCTION static VOID wait_for_cnf_ind(VOID)
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Test whether a confirmation/indication was received and                  */
/* dispatch the received data according to it's layer                       */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  T_PROFI_SERVICE_DESCR      sdb;
  char                       cnf_ind_buffer[512];
  USIGN16                    buffer_len;
  INT16                      ret_val;

FUNCTION_BODY

  for(;;)
  {
    /* check for User Exit */
    buffer_len = sizeof(cnf_ind_buffer);

    
    ret_val = profi_rcv_con_ind((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                                 (VOID FAR *) cnf_ind_buffer,
                                 &buffer_len
                                );
    
    

#ifdef WIN32
   Sleep(1);
#endif

#ifdef _LINUX
   sleep (1);
#endif

    /* Test for valid reception */

    switch (ret_val)
    {
      case NO_CON_IND_RECEIVED:
       // printf("NO_CON_IND_RECEIVED\n");
        return;
      case CON_IND_RECEIVED:
       // printf("CON_IND_RECEIVED\n");
        break;
      default:
        print_if_error (IND,ret_val);
        error_exit ("Receive Error when waiting for CON/IND");
        break;
    }

    /* Dispatch Data */

    switch(sdb.layer)
    {
      case DP_USR:
        dp_ind_con(&sdb,cnf_ind_buffer);
      //  if(global_state != STATE_FMB_BUSPARAMETER_OK)
          print_menu();
        break;

      case FMB_USR:
        fmb_ind_con(&sdb,cnf_ind_buffer);
        
        print_menu();
        break;

      default:
        error_exit ("Unexpected User Layer");
        break;
    }
    fflush(stdout);
  }
  
} /* wait_for_cnf_ind */

/**************************** AUTOMATIC_ACTION ******************************/

FUNCTION static VOID automatic_action(VOID)
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Do all those things like polling the card according to the current state */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  INT16 ret_val = -1;

FUNCTION_BODY

  switch (global_state)
  {
    case STATE_BOARD_OK:
    case STATE_FMB_WAIT_CON:
    case STATE_FMB_WAIT_PAR:
    case STATE_INIT_MASTER_WAIT_CON:
    case STATE_FMB_BUSPARAMETER_OK:
    case STATE_WAIT_FOR_CON:
    case STATE_IDLE:
      /* nothing automatic to be done */
      break;

    case STATE_STARTUP:
      print_version();
      printf("Init Controller:\n");

#if defined (WIN32) || defined (_LINUX)
      ret_val = profi_set_default((USIGN8) board_number,0,0,0);
#else
      ret_val = init_profibus (dpr_base_address, io_address, PB_FALSE);
#endif

      if (ret_val != E_OK)
      {
        print_if_error (REQ,ret_val);
        error_exit ("Could not initialize Controller");
      }

      print_menu ();
      global_state++;
      break;

    case STATE_USER_START:
      print_version();
      print_papi_firmware_version();
      printf("FMB-Configuration: (DP)\n");
      ret_val = fmb_set_configuration();
      global_state++;
      break;

    case STATE_FMB_CONFIGURATION_OK:
      printf("FMB-Busparameter: Baudrate = 93.75 kBaud\n");
      ret_val = fmb_set_busparameter(ts);
      global_state++;
      break;

    case STATE_DP_USER_START:
      print_version();
      printf("DP-Init-Master\n");
      ret_val = dp_init_master_req();
      global_state++;
      break;

    case STATE_DP_INIT_MASTER_OK:
      /* State Stop is sufficient for DP/V1 communication */
      printf("DP-Act-Param-Loc: Stop\n");
      ret_val = dp_act_param_loc_req();
      global_state++;
      break;
  }

  if (ret_val != -1)
  {
    if (ret_val == E_OK)
    {
      printf("Service started successfully\n");
    }
    else
    {
      print_if_error (REQ,ret_val);
      error_exit ("Service was not accepted by the controller\n");
    }
  }
  fflush(stdout);
} /* automatic_action */

/****************************** USER_ACTION *********************************/

FUNCTION static VOID user_action(IN USIGN8 action)
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Do all the users requests                                                */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  INT16   ret_val      = -1;
  USIGN8  slot_number  = 1;
  USIGN8  index        = 1;
  USIGN8  Length;
  int     i;
  USIGN8  data[256];
  unsigned int dummy;
  USIGN16 send_timeout = 1000;

FUNCTION_BODY

  action = tolower(action);

  if (action == 'x')
  {
    error_exit ("Terminated.");
  }

  switch (global_state)
  {
    case STATE_BOARD_OK:
    case STATE_FMB_BUSPARAMETER_OK:
      if (action=='g')
      {
          gateway_flag = 1;
      }
      else
      {
          gateway_flag = 0;
      }
      global_state++;
      break;


    case STATE_STARTUP:
    case STATE_FMB_WAIT_CON:
    case STATE_FMB_WAIT_PAR:
    case STATE_DP_INIT_MASTER_OK:
    case STATE_INIT_MASTER_WAIT_CON:
    case STATE_WAIT_FOR_CON:
      /* no action is possible */
      break;

    case STATE_IDLE:
      switch (action)
      {
        case 'i':
          printf("Send_Timeout [in 10ms] (usually 1000 is a good guess):");
          scanf("%hu",&send_timeout);
          //fgets(line, sizeof (line), stdin);
         // if (strlen(line)) send_timeout = atoi(line);

          print_version();
          ret_val = dp_initiate_req(send_timeout);
          global_state = STATE_WAIT_FOR_CON;
          break;

        case 'w':
          printf("Slot_Number (usually 1 is a good guess):");
          
          /*
          fgets(line, sizeof (line), stdin);
          if (strlen(line)) slot_number = atoi(line);
          */
          scanf("%u", &dummy);
          slot_number = (USIGN8)dummy;

          printf("Index (usually 1 is a good guess):");
          /*
          line[0]=0;
          fgets(line, sizeof (line), stdin);
          if (strlen(line)) index = atoi(line);
          */
          scanf("%u",&dummy);
          index = (USIGN8)dummy;

          print_version();
            
          
          
          global_state = STATE_WAIT_FOR_CON;          
          
          
          printf("enter data length: ");
          scanf("%u",&dummy);
          Length = (USIGN8) dummy;
         // printf("\n");
          for(i=0; i<Length; i++)
          {
            printf("data[%u]: ",i);
            scanf("%u",&dummy);
            data[i] = (USIGN8) dummy;
           // printf("\n");
          }
          
          ret_val = dp_write_req(slot_number,index,Length,data);
          
          break;
          
             
             
        case 'r':
        
          printf("Slot_Number (usually 1 is a good guess):");
          
          /*
          fgets(line, sizeof (line), stdin);
          if (strlen(line)) slot_number = atoi(line);
          */
          scanf("%u", &dummy);
          slot_number = (USIGN8)dummy;

          printf("Index (usually 1 is a good guess):");
          /*
          line[0]=0;
          fgets(line, sizeof (line), stdin);
          if (strlen(line)) index = atoi(line);
          */
          scanf("%u",&dummy);
          index = (USIGN8)dummy;

          print_version();
            
          ret_val = dp_read_req(slot_number,index);

          global_state = STATE_WAIT_FOR_CON;
          break;

        case 'a':
          print_version();
          ret_val = dp_abort_req();
          print_menu ();
          break;

        case 'h':
          print_menu();
          break;

        default:
          break;
      }
      break;

    case STATE_USER_START:
    case STATE_DP_USER_START:
    case STATE_FMB_CONFIGURATION_OK:
      error_exit("User Application fault: State not allowed");
      break;

    default:
      error_exit("User Application fault: Unknown state");
      break;
  }

  if (ret_val != -1)
  {
    if (ret_val == E_OK)
    {
      printf("Service started successfully\n");
      bWaitForInput = 0;
      print_menu();
    }
    else
    {
      global_state = STATE_IDLE;
      printf("Could not start service\n");
      print_if_error (REQ,ret_val);
      print_menu();
    }
  }
  fflush(stdout);
} /* user_action */

/********************************** DPV1_TASK *******************************/

FUNCTION static VOID dpv1_task(VOID )
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* The program's main loop                                                  */
/* ------------------------------------------------------------------------ */
{

  char c[100];


  for (;;)
  {
    automatic_action();
    wait_for_cnf_ind();
    if(bWaitForInput)
    {
      bWaitForInput = 0;
      scanf("%s",c);
      user_action((char)c[0]);
    }


  }
} /* dpv1_task */

/****************************************************************************/
/**** COMMAND-LINE **********************************************************/
/****************************************************************************/

FUNCTION static VOID check_options(IN    char    options [])
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Analysis of the Users command line options                               */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

 char FAR*    token_ptr;

FUNCTION_BODY

  token_ptr = strtok (options, " \t\r");

  if (token_ptr != NULL)
  {
#if defined (WIN32) || defined (_LINUX)
    if (strncmp (token_ptr, "-b", 2) == 0)
    {
      if (sscanf (token_ptr, "-b%hi", &board_number) == 1)
      {
        if(!((board_number > 0 ) && (board_number <= 9 )))
        {
          board_number = BOARD_NUMBER;
        }
      }
      else
      {
        board_number = BOARD_NUMBER;
      }
    }
#else
    if (strncmp (token_ptr, "-a", 2) == 0)
    {
      if (sscanf (token_ptr, "-a0x%8lx", &dpr_base_address) != 1)
      {
        if (sscanf (token_ptr, "-a%10lu", &dpr_base_address) != 1)
        {
          dpr_base_address = (USIGN32) H_DPR_BASE_ADDRESS;
        }
      }
    }
    else if (strncmp (token_ptr, "-i", 2) == 0)
    {
      if (sscanf (token_ptr, "-i0x%4x", &io_address) != 1)
      {
        if (sscanf (token_ptr, "-i%5u", &io_address) != 1)
        {
          io_address = (USIGN16) IO_ADDRESS;
        }
      }
    }
#endif
    else if (strncmp (token_ptr, "-s", 2) == 0)
    {
      if (sscanf (token_ptr, "-s0x%2hx", &remote_address) != 1)
      {
        if (sscanf (token_ptr, "-s%3hu", &remote_address) != 1)
        {
          remote_address = DEFAULT_SLAVE;
        }
      }
      if (remote_address==DEFAULT_MASTER)
      {
        ts = DEFAULT_SLAVE;
      }
      else
      {
        ts = DEFAULT_MASTER;
      }
    }
    else if ( (strcmp (token_ptr, "-?") == 0) ||
              (strcmp (token_ptr, "-h") == 0) )
    {
      print_version ();
      print_usage ();
      error_exit("");
    }
    else
    {
      print_version ();
      printf ("error: unknown command line parameter \"%s\" \n\n", token_ptr);
      print_usage ();
      error_exit("");
    }

    token_ptr = strtok (NULL, " \t\r");

  } /* if token_ptr */
  printf("using board %u, slave %u\n",board_number, ts);
  fflush(stdout);
} /* check_options */

/************************ SET_DEFAULT_OPTIONS *******************************/

FUNCTION static VOID set_default_options(VOID)
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* Initialize command-line Values to Default                                */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

FUNCTION_BODY

#if defined (WIN32) || defined (_LINUX)
  board_number      = BOARD_NUMBER;
#else
  dpr_base_address  = (USIGN32) H_DPR_BASE_ADDRESS;
  io_address        = (USIGN16) IO_ADDRESS;
#endif

  ts                = DEFAULT_MASTER;
  remote_address    = DEFAULT_SLAVE;

} /* set_default_options */

/****************************************************************************/
/**** MAIN * MAIN * MAIN ****************************************************/
/****************************************************************************/

FUNCTION extern int main(IN  int  argc,IN  char **argv)
/* ------------------------------------------------------------------------ */
/* FUNCTIONAL_DESCRIPTION:                                                  */
/*                                                                          */
/* DP/V1-Demo Program for DOS16                                             */
/* ------------------------------------------------------------------------ */
{
LOCAL_VARIABLES

  int       i;

#if defined (WIN16) || defined (WIN32)
  char      dummy [_MAX_PATH];
#endif

FUNCTION_BODY

#if defined (WIN16) || defined (WIN32)
  _splitpath (argv [0], dummy, dummy, exe_fname, dummy);
#endif

  set_default_options ();                      /* program default parameters */

  /* -- check command line ------------------------------------------------- */
  for (i = 1; i < argc; i++)
  {
    check_options (argv [i]);
  }

  global_state = STATE_STARTUP;
  dpv1_task();
  
  return (0);
  
} /* main */

/****************************************************************************/
/**** EOF *******************************************************************/
/****************************************************************************/
